package com.cmmap.api.maps;

import com.cmmap.api.maps.model.CameraPosition;
import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.LatLngBounds;
import com.cmmap.internal.maps.KCameraUpdate;
import com.cmmap.internal.maps.KCameraUpdateFactory;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图可视范围更新创建类。
 * <p>该类用于创建{@link CameraUpdate}对象，
 * 主要用于用户调用{@link Map#moveCamera}和{@link Map#animateCamera}修改地图可视范围。</p>
 */
public final class CameraUpdateFactory {
	/**
	 * 返回一个{@link CameraUpdate}对象，只改变地图中心点。
	 * @param latLng 地图中心点坐标
	 * @return 一个包含地图中心点坐标改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate changeLatLng(LatLng latLng) {
			KLatLng kLatLng = new KLatLng(latLng);
			KCameraUpdate kUpdate = KCameraUpdateFactory.changeLatLng(kLatLng);
			CameraUpdate update = new CameraUpdate(kUpdate);
			return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，只改变旋转角度。
	 * @param bearing 地图旋转角度，正北方向到地图方向逆时针旋转的角度，单位：角度，范围：[0,360)
	 * @return 一个包含旋转角度改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate changeBearing(float bearing) {
		KCameraUpdate kUpdate = KCameraUpdateFactory.changeBearing(bearing);
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，只改变地图倾斜度。
	 * @param tilt 地图倾斜度，单位：角度，范围：[0,45]。
	 * @return 一个包含地图倾斜度改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate changeTilt(float tilt) {
		KCameraUpdate kUpdate = KCameraUpdateFactory.changeTilt(tilt);
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个包含缩放级别增大的{@link CameraUpdate}对象，调用此方法一次缩放级别加大一级，也就是屏幕距离地面更近一级。
	 * @return 一个包含缩放级别改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate zoomIn() {
		KCameraUpdate kUpdate = KCameraUpdateFactory.zoomIn();
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个包含缩放级别减小的{@link CameraUpdate}对象，调用此方法一次缩放级别的减小一级，也就是屏幕距离地面更远一级。
	 * @return 一个包含缩放级别改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate zoomOut() {
		KCameraUpdate kUpdate = KCameraUpdateFactory.zoomOut();
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个包含缩放级别改变的{@link CameraUpdate}对象。
	 * @param zoom 地图缩放级别
	 * @return 一个包含地图缩放级别改变的{@link CameraUpdate}对象
	 */
	public static CameraUpdate zoomTo(float zoom) {
		KCameraUpdate kUpdate = KCameraUpdateFactory.zoomTo(zoom);
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，包括地图中心点坐标以及缩放级别，地图旋转角度和倾斜角度为0。
	 * @param latLng 地图中心点坐标
	 * @param zoom 地图缩放级别
	 * @return 一个包含地图中心点坐标和缩放级的@link CameraUpdate}对象
	 */
	public static CameraUpdate newLatLngZoom(LatLng latLng, float zoom) {
		KLatLng kLatLng = new KLatLng(latLng);
		KCameraUpdate kUpdate = KCameraUpdateFactory.newLatLngZoom(kLatLng, zoom);
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，包含一个经纬度限制区域，并且是最大可能的缩放级别，地图旋转角度和倾斜角度为0。
	 * @param bounds 经纬度限制区域
	 * @param leftPadding 经纬度限制区域和地图视图之间的左边空白距离，单位像素
	 * @param rightPadding 经纬度限制区域和地图视图之间的右边空白距离，单位像素
	 * @param topPadding 经纬度限制区域和地图视图之间的上边空白距离，单位像素
	 * @param bottomPadding 经纬度限制区域和地图视图之间的下边空白距离，单位像素
	 * @return 一个包含经纬度限制区域的{@link CameraUpdate}对象
	 */
	public static CameraUpdate newLatLngBounds(LatLngBounds bounds,
			int leftPadding, int rightPadding, int topPadding, int bottomPadding) {
		KCameraUpdate kUpdate = KCameraUpdateFactory.newLatLngBounds(bounds, leftPadding, rightPadding, topPadding, bottomPadding);
		return new CameraUpdate(kUpdate);
	}

	/**
	 * 返回一个定义了地图可视范围的{@link CameraUpdate}对象。
	 * @param camera 地图可视范围对象。
	 * @return 一个包含地图可视范围的{@link CameraUpdate}对象
	 */
	public static CameraUpdate newCameraPosition(CameraPosition camera) {
		KCameraUpdate kUpdate = KCameraUpdateFactory.newCameraPosition(camera);
		CameraUpdate update = new CameraUpdate(kUpdate);
		return update;
	}
}
