package com.cmmap.api.maps;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.MotionEvent;
import android.view.View;

import com.cmmap.api.maps.model.Arc;
import com.cmmap.api.maps.model.ArcOptions;
import com.cmmap.api.maps.model.CameraPosition;
import com.cmmap.api.maps.model.Circle;
import com.cmmap.api.maps.model.CircleOptions;
import com.cmmap.api.maps.model.Event;
import com.cmmap.api.maps.model.GroundOverlay;
import com.cmmap.api.maps.model.GroundOverlayOptions;
import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.Marker;
import com.cmmap.api.maps.model.MarkerOptions;
import com.cmmap.api.maps.model.MyTrafficStyle;
import com.cmmap.api.maps.model.NavigateArrow;
import com.cmmap.api.maps.model.NavigateArrowOptions;
import com.cmmap.api.maps.model.Poi;
import com.cmmap.api.maps.model.Polygon;
import com.cmmap.api.maps.model.PolygonOptions;
import com.cmmap.api.maps.model.Polyline;
import com.cmmap.api.maps.model.PolylineOptions;
import com.cmmap.api.maps.model.TileOverlay;
import com.cmmap.api.maps.model.TileOverlayOptions;
import com.cmmap.internal.driver.base.DataOperator;
import com.cmmap.internal.maps.KMap;
import com.cmmap.internal.maps.model.KArc;
import com.cmmap.internal.maps.model.KArcOptions;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KCircle;
import com.cmmap.internal.maps.model.KCircleOptions;
import com.cmmap.internal.maps.model.KGroundOverlay;
import com.cmmap.internal.maps.model.KGroundOverlayOptions;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KMarkerOptions;
import com.cmmap.internal.maps.model.KNavigateArrow;
import com.cmmap.internal.maps.model.KNavigateArrowOptions;
import com.cmmap.internal.maps.model.KOverlayManager;
import com.cmmap.internal.maps.model.KPoi;
import com.cmmap.internal.maps.model.KPolygon;
import com.cmmap.internal.maps.model.KPolygonOptions;
import com.cmmap.internal.maps.model.KPolyline;
import com.cmmap.internal.maps.model.KPolylineOptions;
import com.cmmap.internal.maps.model.KTileOverlay;
import com.cmmap.internal.maps.model.KTileOverlayOptions;

/**
 * 地图对象。
 * <p>定义地图对象的操作方法与接口。</p>
 */
public class Map {
	/** 地图模式：正常模式 */
	public static final int MAP_TYPE_NORMAL = 0;
	/** 地图模式：黑夜地图 */
	public static final int MAP_TYPE_NIGHT = 1;
	/** 地图模式：卫星底图 */
	public static final int MAP_TYPE_SATELLITE = 2;
	/** 地图模式：导航模式 */
	public static final int MAP_TYPE_NAVI = 3;
	/** 地图模式：公交地图 */
	public static final int MAP_TYPE_BUS = 4;
	/** 地图模式：路线概览地图 */
	public static final int MAP_TYPE_RP = 5;
	/** 地图模式：黑夜导航地图 */
	public static final int MAP_TYPE_NAVI_NIGHT = 6;
	
	private KMap mKMap;
	private UiSettings mUiSettings;
	private Projection mProjection;
	
	
	private OnMapLoadedListener mOnMapLoadedListener;
	private OnMapPropUpdateListener mOnMapPropUpdateListener;
	private OnMapTouchListener  mOnMapTouchListener;
	private OnMapClickListener  mOnMapClickListener;
	private OnMapLongClickListener mOnMapLongClickListener;
	private OnPOIClickListener mOnPOIClickListener;
	private OnEventClickListener mOnEventClickListener;
	private OnArcClickListener mOnArcClickListener;
	private OnArcDragListener mOnArcDragListener;
	private OnCircleClickListener mOnCircleClickListener;
	private OnCircleDragListener mOnCircleDragListener;
	private OnPolylineClickListener mOnPolylineClickListener;
	private OnPolylineDragListener mOnPolylineDragListener;
	private OnPolygonClickListener mOnPolygonClickListener;
	private OnPolygonDragListener mOnPolygonDragListener;
	private OnGroundOverlayClickListener mOnGroundOverlayClickListener;
	private OnGroundOverlayDragListener mOnGroundOverlayDragListener;
	private OnMarkerClickListener mOnMarkerClickListener;
	private OnMarkerDragListener mOnMarkerDragListener;
	private InfoWindowAdapter mInfoWindowAdapter;
	private OnInfoWindowClickListener mOnInfoWindowClickListener;
	private OnCameraChangeListener mOnCameraChangeListener;
	private CancelableCallback mCancelableCallback;
	
	private ArrayList<Object> overlays = new ArrayList<Object>();
	private KOverlayManager.OnOverlayListener mOnOverlayListener = new KOverlayManager.OnOverlayListener() {
		
		@Override
		public void onRemove(Object overlay) {
			if (overlay != null) {
				overlays.remove(overlay);
			}
		}
	};

	/**
	 * 内部构造函数，不公开提供
	 */
	Map() {
	}
	
	Map(KMap map) {
		if (map != null) {
			mKMap = map;
			mUiSettings = new UiSettings(map.getUiSettings());
			mProjection = new Projection(map.getProjection());
		}
	}

	/**
	 * 地图加载完成后的回调接口。
	 */
	public static interface OnMapLoadedListener {
		/**
		 * 地图加载完成后回调此方法。
		 */
		void onMapLoaded();
		
		/**
		 * 地图加载失败后回调此方法。
		 */
		void onMapLoadFailure();
	}
	
	/**
	 * 地图属性更新回调接口。当用户设置该回调接口时，所有的接口函数立刻回调一次。
	 */
	public static interface OnMapPropUpdateListener {
		/**
		 * 地图版本更新后回调此方法。
		 * @param mapVersion 地图版本号。版本号由一个主版本号和一个副版本号组成，例如"1.2"、"2.3"等。
		 */
		void onMapVersionUpdate(String mapVersion);
	}

	/**
	 * 用户触摸地图时的回调接口。
	 */
	public static interface OnMapTouchListener {
		/**
		 * 用户触摸地图时回调此方法 。
		 * @param event 地图触摸事件
		 */
		void onTouch(MotionEvent event);
	}

	/**
	 * 当用户点击地图时的回调接口。
	 */
	public static interface OnMapClickListener {
		/**
		 * 当用户点击地图时回调此方法，如果点击在覆盖物上，并且该覆盖物相应了点击事件，则不会触发此方法。
		 * @param point 用户所点击的地图坐标
		 */
		void onMapClick(LatLng point);
	}

	/**
	 * 户长按地图时的回调接口。
	 */
	public static interface OnMapLongClickListener {
		/**
		 * 用户长按地图时回调此方法。
		 * @param point 用户点击的地图经纬度位置
		 */
		void onMapLongClick(LatLng point);
	}

	/**
	 * 地图底图POI点被选中时的回调接口。
	 */
	public static interface OnPOIClickListener {
		/**
		 * 地图底图POI点被选中时回调此方法 。
		 * @param poi 点击的地图POI对象
		 */
		void onPOIClick(Poi poi);
	}
	
	/**
	 * 地图底图路况事件点被选中时的回调接口。
	 */
	public static interface OnEventClickListener {
		/**
		 * 地图底图路况事件点被选中时回调此方法 。
		 * @param event 点击的地图路况事件对象
		 */
		void onEventClick(Event event);
	}	

	/**
	 * 弧形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnArcClickListener {
		/**
		 * 弧形覆盖物对象被点击时调用此方法。
		 * @param arc 被点击的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcClick(Arc arc, LatLng clickPoint);
	}

	/**
	 * 弧形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnArcDragListener {
		/**
		 * 当弧形覆盖物对象开始被拖动时回调此方法。
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDragStart(Arc arc, LatLng clickPoint);

		/**
		 * 在弧形覆盖物对象拖动过程当中回调此方法。
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDrag(Arc arc, LatLng clickPoint); 

		/**
		 * 当弧形覆盖物对象拖动完成后回调此方法。 
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDragEnd(Arc arc, LatLng clickPoint); 
	}

	/**
	 * 圆形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnCircleClickListener {
		/**
		 * 圆形覆盖物对象被点击时调用此方法。
		 * @param circle 被点击的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleClick(Circle circle, LatLng clickPoint);
	}

	/**
	 * 圆形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnCircleDragListener {
		/**
		 * 当圆形覆盖物对象开始被拖动时回调此方法。
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDragStart(Circle circle, LatLng clickPoint);

		/**
		 * 在圆形覆盖物对象拖动过程当中回调此方法。
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDrag(Circle circle, LatLng clickPoint); 

		/**
		 * 当圆形覆盖物对象拖动完成后回调此方法。 
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDragEnd(Circle circle, LatLng clickPoint); 
	}

	/**
	 * 线段覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolylineClickListener {
		/**
		 * 线段覆盖物对象被点击时调用此方法。
		 * @param polyline 被点击的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineClick(Polyline polyline, LatLng clickPoint);
	}

	/**
	 * 线段覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolylineDragListener {
		/**
		 * 当线段覆盖物对象开始被拖动时回调此方法。
		 * <br>线段覆盖物对象的位置可以通过{@link Polyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDragStart(Polyline polyline, LatLng clickPoint);

		/**
		 * 在线段覆盖物对象拖动过程当中回调此方法。
		 * <br>线段覆盖物对象的位置可以通过{@link Polyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDrag(Polyline polyline, LatLng clickPoint); 

		/**
		 * 当线段覆盖物对象拖动完成后回调此方法。 
		 * <br>线段覆盖物对象的位置可以通过{@link Polyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDragEnd(Polyline polyline, LatLng clickPoint); 
	}

	/**
	 * 多边形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolygonClickListener {
		/**
		 * 多边形覆盖物对象被点击时调用此方法。
		 * @param polygon 被点击的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonClick(Polygon polygon, LatLng clickPoint);
	}

	/**
	 * 多边形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolygonDragListener {
		/**
		 * 当多边形覆盖物对象开始被拖动时回调此方法。
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDragStart(Polygon polygon, LatLng clickPoint);

		/**
		 * 在多边形覆盖物对象拖动过程当中回调此方法。
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDrag(Polygon polygon, LatLng clickPoint); 

		/**
		 * 当多边形覆盖物对象拖动完成后回调此方法。 
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDragEnd(Polygon polygon, LatLng clickPoint); 
	}

	/**
	 * 图片覆盖物对象被点击时的回调接口。
	 */
	public static interface OnGroundOverlayClickListener {
		/**
		 * 图片覆盖物对象被点击时调用此方法。
		 * @param groundOverlay 被点击的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayClick(GroundOverlay groundOverlay, LatLng clickPoint);
	}

	/**
	 * 图片覆盖物对象被点击时的回调接口。
	 */
	public static interface OnGroundOverlayDragListener {
		/**
		 * 当图片覆盖物对象开始被拖动时回调此方法。
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDragStart(GroundOverlay groundOverlay, LatLng clickPoint);

		/**
		 * 在图片覆盖物对象拖动过程当中回调此方法。
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDrag(GroundOverlay groundOverlay, LatLng clickPoint); 

		/**
		 * 当图片覆盖物对象拖动完成后回调此方法。 
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDragEnd(GroundOverlay groundOverlay, LatLng clickPoint); 
	}

	/**
	 * 标记覆盖物对象被点击时的回调接口。
	 */
	public static interface OnMarkerClickListener {
		/**
		 * 标记覆盖物对象被点击时回调此方法 。
		 * <br>不管返回结果是什么，SDK都不会主动显示该覆盖物的信息窗口，用户要想显示信息窗口，需要主动调用{@link Marker#showInfoWindow}
		 * @param marker 被点击的标记覆盖物对象
		 * @return 用户是否处理了该事件
		 */
		boolean onMarkerClick(Marker marker);
	}

	/**
	 * 标记覆盖物对象拖动时的回调接口。
	 */
	public static interface OnMarkerDragListener {
		/**
		 * 当标记覆盖物对象开始被拖动时回调此方法。
		 * <br>标记覆盖物对象的位置可以通过{@link Marker#getPosition}方法返回。
		 * @param marker 被拖动的标记覆盖物对象
		 */
		void onMarkerDragStart(Marker marker);

		/**
		 * 在标记覆盖物对象拖动过程当中回调此方法。
		 * <br>标记覆盖物对象的位置可以通过{@link Marker#getPosition}方法返回。
		 * @param marker 被拖动的标记覆盖物对象
		 */
		void onMarkerDrag(Marker marker); 

		/**
		 * 当标记覆盖物对象拖动完成后回调此方法。 
		 * <br>标记覆盖物对象的位置可以通过{@link Marker#getPosition}方法返回。
		 * @param marker 被拖动的标记覆盖物对象
		 */
		void onMarkerDragEnd(Marker marker); 
	}

	/**
	 * 针对具体的标记覆盖物的个性化定制的信息窗口的回调接口。
	 */
	public static interface InfoWindowAdapter {
		/**
		 * 返回针对具体的标记覆盖物的个性化定制的信息窗口的视图。
		 * @param marker 与该信息窗口对应的标记覆盖物对象
		 * @return 针对该标记覆盖物的个性化定制的信息窗口的视图，
		 * 如果返回null，SDK将以默认的信息窗口样式显示，信息窗口内容为{@link Marker#getTitle}和{@link Marker#getSnippet}
		 */
		View getInfoWindow(Marker marker);
	}

	/**
	 * 信息窗口点击的回调接口。
	 */
	public static interface OnInfoWindowClickListener {
		/**
		 * 当信息窗口被点击时回调此方法 。
		 * @param marker 被点击的信息窗口的标记覆盖物对象
		 */
		void onInfoWindowClick(Marker marker);
	}

	/**
	 * 地图可视范围改变时的回调接口。
	 * <br>
	 * 如果是用户手势操作或者通过{@link #animateCamera}操作导致地图可视范围变化，
	 * 变化过程中调用onCameraChange函数，变化完成后调用onCameraChangeFinish。
	 * <br>
	 * 其他不存在中间过程的情况，同时调用onCameraChange和onCameraChangeFinish。
	 */
	public static interface OnCameraChangeListener {
		/**
		 * 地图可视范围改变过程中回调此方法。
		 * @param position 当前地图可视范围对象
		 */
		void onCameraChange(CameraPosition position);

		/**
		 * 地图可视范围改变完成回调此方法。
		 * @param postion 当前地图可视范围对象
		 */
		void onCameraChangeFinish(CameraPosition postion);
	}
	
	/**
	 * 一个任务完成或取消时的回调接口。
	 * <br>
	 * 这是一个通用的接口，典型的用于{@link #animateCamera}调用。
	 */
	public static interface CancelableCallback {
		/**
		 * 当一个任务完成时回调此方法
		 */
		void onFinish();
		
		/**
		 * 当一个任务被终止时回调此方法
		 */
		void onCancel();
	}
	
	/**
	 * 地图截屏监听接口
	 */
	public static interface OnMapScreenShotListener {
	    /**
	     * 地图截屏回调此方法。
	     * @param bitmap 截屏返回的bitmap对象
	     */
	    void onMapScreenShot(Bitmap bitmap);
	    
	    /**
	     * 地图截屏回调此方法，并返回截屏一瞬间地图是否渲染完成。
	     * @param bitmap 截屏返回的bitmap对象
	     * @param status 地图渲染状态， 1：地图渲染完成， 0：未渲染完成
	     */
	    void onMapScreenShot(Bitmap bitmap, int status);
	}

	/**
	 * 设置地图加载完成回调接口。
	 * @param listener 地图加载完成回调接口
	 */
	public void setOnMapLoadedListener(OnMapLoadedListener listener) {
		mOnMapLoadedListener = listener;
		if (listener != null) {
			if (mKMap == null) {
				listener.onMapLoadFailure();
			} else {				
				mKMap.setOnMapLoadedListener(new KMap.OnMapLoadedListener() {
					
					@Override
					public void onMapLoaded() {
						if(mOnMapLoadedListener != null) {
							mOnMapLoadedListener.onMapLoaded();
						}
					}
					
					@Override
					public void onMapLoadFailure() {
						if (mOnMapLoadedListener != null) {
							mOnMapLoadedListener.onMapLoadFailure();
						}
						
					}
				});
			}
		} else {
			mKMap.setOnMapLoadedListener(null);
		}
		
	}
	
	/**
	 * 设置地图属性更新回调接口。
	 * @param listener 地图属性更新回调接口
	 */
	public void setOnMapPropUpdateListener(OnMapPropUpdateListener listener) {
		mOnMapPropUpdateListener = listener;
		if (listener != null) {
			mKMap.setOnMapPropUpdateListener(new KMap.OnMapPropUpdateListener() {
				
				@Override
				public void onMapVersionUpdate(String mapVersion) {
					mOnMapPropUpdateListener.onMapVersionUpdate(mapVersion);
				}
			});
		} else {
			mKMap.setOnMapPropUpdateListener(null);
		}
	}

	/**
	 * 设置地图触摸回调接口。
	 * @param listener 地图触摸回调接口
	 */
	public void setOnMapTouchListener(OnMapTouchListener listener) {
		mOnMapTouchListener = listener;
		if (listener != null) {
			mKMap.setOnMapTouchListener(new KMap.OnMapTouchListener() {
				
				@Override
				public void onTouch(MotionEvent event) {
					if (mOnMapTouchListener != null) {
						mOnMapTouchListener.onTouch(event);
					}	
				}
			});
		} else {
			mKMap.setOnMapTouchListener(null);
		}
	}

	/**
	 * 设置地图点击回调接口。
	 * @param listener 地图点击回调接口
	 */
	public void setOnMapClickListener(OnMapClickListener listener) {
		mOnMapClickListener = listener;
		if (listener != null) {
			mKMap.setOnMapClickListener(new KMap.OnMapClickListener() {
				
				@Override
				public void onMapClick(KLatLng point) {
					if (mOnMapClickListener != null) {
						mOnMapClickListener.onMapClick(point.toLatLng());
					}
				}
			});
		} else {
			mKMap.setOnMapClickListener(null);
		}
	}

	/**
	 * 设置地图长按回调接口。
	 * @param listener 地图长按回调接口
	 */
	public void setOnMapLongClickListener(OnMapLongClickListener listener) {
		mOnMapLongClickListener  = listener;
		if (listener != null) {
			mKMap.setOnMapLongClickListener(new KMap.OnMapLongClickListener() {
				
				@Override
				public void onMapLongClick(KLatLng point) {
					if (mOnMapLongClickListener != null) {
						mOnMapLongClickListener.onMapLongClick(point.toLatLng());
					}
				}
			});
		} else {
			mKMap.setOnMapLongClickListener(null);
		}
		
	}

	/**
	 * 设置地图底图POI点击选中回调接口。
	 * @param listener 地图底图POI点击选中回调接口
	 */
	public void setOnPOIClickListener(OnPOIClickListener listener) {
		mOnPOIClickListener = listener;
		if (listener != null) {
			mKMap.setOnPOIClickListener(new KMap.OnPOIClickListener() {
				
				@Override
				public void onPOIClick(KPoi poi) {
					if (mOnPOIClickListener != null) {
						mOnPOIClickListener.onPOIClick(poi.toPoi());
					}
				}
			});
		} else {
			mKMap.setOnPOIClickListener(null);
		}
	}
	
	/**
	 * 设置地图底图路况事件点击选中回调接口。
	 * @param listener 地图底图路况事件点击选中回调接口
	 */
	public void setOnEventClickListener(OnEventClickListener listener) {
		mOnEventClickListener = listener;
		if (listener != null) {
			mKMap.setOnEventClickListener(new KMap.OnEventClickListener() {
				
				@Override
				public void onEventClick(KPoi poi) {
					if (mOnEventClickListener != null) {
						mOnEventClickListener.onEventClick(poi.toEvent());
					}
				}
			});
		} else {
			mKMap.setOnEventClickListener(null);
		}
	}

	/**
	 * 设置弧形覆盖物点击回调接口。
	 * @param listener 弧形覆盖物点击回调接口
	 */
	public void setOnArcClickListener(OnArcClickListener listener) {
		mOnArcClickListener = listener;
		if (listener != null) {
			mKMap.setOnArcClickListener(new KMap.OnArcClickListener() {

				@Override
				public void onArcClick(KArc arc, KLatLng clickPoint) {
					if (mOnArcClickListener != null && arc != null && arc.getParent() != null && clickPoint != null) {
						mOnArcClickListener.onArcClick(arc.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnArcClickListener(null);
		}
	}

	/**
	 * 设置弧形覆盖物拖动回调接口。
	 * @param listener 弧形覆盖物拖动回调接口
	 */
	public void setOnArcDragListener(OnArcDragListener listener) {
		mOnArcDragListener = listener;
		if (listener != null) {
			mKMap.setOnArcDragListener(new KMap.OnArcDragListener() {
				
				@Override
				public void onArcDragStart(KArc arc, KLatLng clickPoint) {
					if (mOnArcDragListener != null && arc != null && arc.getParent() != null && clickPoint != null) {
						mOnArcDragListener.onArcDragStart(arc.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onArcDragEnd(KArc arc, KLatLng clickPoint) {
					if (mOnArcDragListener != null && arc != null && arc.getParent() != null && clickPoint != null) {
						mOnArcDragListener.onArcDragEnd(arc.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onArcDrag(KArc arc, KLatLng clickPoint) {
					if (mOnArcDragListener != null && arc != null && arc.getParent() != null && clickPoint != null) {
						mOnArcDragListener.onArcDrag(arc.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnArcClickListener(null);
		}
	}

	/**
	 * 设置圆形覆盖物点击回调接口。
	 * @param listener 圆形覆盖物点击回调接口
	 */
	public void setOnCircleClickListener(OnCircleClickListener listener) {
		mOnCircleClickListener = listener;
		if (listener != null) {
			mKMap.setOnCircleClickListener(new KMap.OnCircleClickListener() {
				
				@Override
				public void onCircleClick(KCircle circle, KLatLng clickPoint) {
					if (mOnCircleClickListener != null && circle != null && circle.getParent() != null && clickPoint != null) {
						mOnCircleClickListener.onCircleClick(circle.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnCircleClickListener(null);
		}
	}

	/**
	 * 设置圆形覆盖物拖动回调接口。
	 * @param listener 圆形覆盖物拖动回调接口
	 */
	public void setOnCircleDragListener(OnCircleDragListener listener) {
		mOnCircleDragListener = listener;
		if (listener != null) {
			mKMap.setOnCircleDragListener(new KMap.OnCircleDragListener() {
				
				@Override
				public void onCircleDragStart(KCircle circle, KLatLng clickPoint) {
					if (mOnCircleDragListener != null && circle != null && circle.getParent() != null && clickPoint != null) {
						mOnCircleDragListener.onCircleDragStart(circle.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onCircleDragEnd(KCircle circle, KLatLng clickPoint) {
					if (mOnCircleDragListener != null && circle != null && circle.getParent() != null && clickPoint != null) {
						mOnCircleDragListener.onCircleDragEnd(circle.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onCircleDrag(KCircle circle, KLatLng clickPoint) {
					if (mOnCircleDragListener != null && circle != null && circle.getParent() != null && clickPoint != null) {
						mOnCircleDragListener.onCircleDrag(circle.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnCircleDragListener(null);
		}
	}

	/**
	 * 设置线段覆盖物点击回调接口。
	 * @param listener 线段覆盖物点击回调接口
	 */
	public void setOnPolylineClickListener(OnPolylineClickListener listener) {
		mOnPolylineClickListener = listener;
		if (listener != null) {
			mKMap.setOnPolylineClickListener(new KMap.OnPolylineClickListener() {

				@Override
				public void onPolylineClick(KPolyline polyline, KLatLng clickPoint) {
					if (mOnPolylineClickListener != null && polyline != null && polyline.getParent() != null && clickPoint != null) {
						mOnPolylineClickListener.onPolylineClick(polyline.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnPolylineClickListener(null);
		}
	}

	/**
	 * 设置线段覆盖物拖动回调接口。
	 * @param listener 线段覆盖物拖动回调接口
	 */
	public void setOnPolylineDragListener(OnPolylineDragListener listener) {
		mOnPolylineDragListener = listener;
		if (listener != null) {
			mKMap.setOnPolylineDragListener(new KMap.OnPolylineDragListener() {
				
				@Override
				public void onPolylineDragEnd(KPolyline polyline, KLatLng clickPoint) {
					if (mOnPolylineDragListener != null && polyline != null && polyline.getParent() != null && clickPoint != null) {
						mOnPolylineDragListener.onPolylineDragEnd(polyline.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onPolylineDrag(KPolyline polyline, KLatLng clickPoint) {
					if (mOnPolylineDragListener != null && polyline != null && polyline.getParent() != null && clickPoint != null) {
						mOnPolylineDragListener.onPolylineDrag(polyline.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onPolylineDragStart(KPolyline polyline, KLatLng clickPoint) {
					if (mOnPolylineDragListener != null && polyline != null && polyline.getParent() != null && clickPoint != null) {
						mOnPolylineDragListener.onPolylineDragStart(polyline.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnPolygonDragListener(null);
		}
	}

	/**
	 * 设置多边形覆盖物点击回调接口。
	 * @param listener 多边形覆盖物点击回调接口
	 */
	public void setOnPolygonClickListener(OnPolygonClickListener listener) {
		mOnPolygonClickListener = listener;
		if (listener != null) {
			mKMap.setOnPolygonClickListener(new KMap.OnPolygonClickListener() {

				@Override
				public void onPolygonClick(KPolygon polygon, KLatLng clickPoint) {
					if (mOnPolygonClickListener != null && polygon != null && polygon.getParent() != null && clickPoint != null) {
						mOnPolygonClickListener.onPolygonClick(polygon.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnPolygonClickListener(null);
		}
	}

	/**
	 * 设置多边形覆盖物拖动回调接口。
	 * @param listener 多边形覆盖物拖动回调接口
	 */
	public void setOnPolygonDragListener(OnPolygonDragListener listener) {
		mOnPolygonDragListener = listener;
		if (listener != null) {
			mKMap.setOnPolygonDragListener(new KMap.OnPolygonDragListener() {
				
				@Override
				public void onPolygonDragStart(KPolygon polygon, KLatLng clickPoint) {
					if (mOnPolygonDragListener != null && polygon != null && polygon.getParent() != null && clickPoint != null) {
						mOnPolygonDragListener.onPolygonDragStart(polygon.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onPolygonDragEnd(KPolygon polygon, KLatLng clickPoint) {
					if (mOnPolygonDragListener != null && polygon != null && polygon.getParent() != null && clickPoint != null) {
						mOnPolygonDragListener.onPolygonDragEnd(polygon.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onPolygonDrag(KPolygon polygon, KLatLng clickPoint) {
					if (mOnPolygonDragListener != null && polygon != null && polygon.getParent() != null && clickPoint != null) {
						mOnPolygonDragListener.onPolygonDrag(polygon.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnPolygonDragListener(null);
		}
	}

	/**
	 * 设置图片覆盖物点击回调接口。
	 * @param listener 图片覆盖物点击回调接口
	 */
	public void setOnGroundOverlayClickListener(OnGroundOverlayClickListener listener) {
		mOnGroundOverlayClickListener = listener;
		if (listener != null) {
			mKMap.setOnGroundOverlayClickListener(new KMap.OnGroundOverlayClickListener() {
				
				@Override
				public void onGroundOverlayClick(KGroundOverlay groundOverlay,KLatLng clickPoint) {
					if (mOnGroundOverlayClickListener != null && groundOverlay != null && groundOverlay.getParent() != null && clickPoint != null) {
						mOnGroundOverlayClickListener.onGroundOverlayClick(groundOverlay.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnGroundOverlayClickListener(null);
		}
	}

	/**
	 * 设置图片覆盖物拖动回调接口。
	 * @param listener 图片覆盖物拖动回调接口
	 */
	public void setOnGroundOverlayDragListener(OnGroundOverlayDragListener listener) {
		mOnGroundOverlayDragListener = listener;
		if (listener != null) {
			mKMap.setOnGroundOverlayDragListener(new KMap.OnGroundOverlayDragListener() {
				
				@Override
				public void onGroundOverlayDragStart(KGroundOverlay groundOverlay, KLatLng clickPoint) {
					if (mOnGroundOverlayDragListener != null && groundOverlay != null && groundOverlay.getParent() != null && clickPoint != null) {
						mOnGroundOverlayDragListener.onGroundOverlayDragStart(groundOverlay.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onGroundOverlayDragEnd(KGroundOverlay groundOverlay, KLatLng clickPoint) {
					if (mOnGroundOverlayDragListener != null && groundOverlay != null && groundOverlay.getParent() != null && clickPoint != null) {
						mOnGroundOverlayDragListener.onGroundOverlayDragEnd(groundOverlay.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
				
				@Override
				public void onGroundOverlayDrag(KGroundOverlay groundOverlay, KLatLng clickPoint) {
					if (mOnGroundOverlayDragListener != null && groundOverlay != null && groundOverlay.getParent() != null && clickPoint != null) {
						mOnGroundOverlayDragListener.onGroundOverlayDrag(groundOverlay.getParent(), new LatLng(clickPoint.latitude, clickPoint.longitude));
					}
				}
			});
		} else {
			mKMap.setOnGroundOverlayDragListener(null);
		}
	}

	/**
	 * 设置标记覆盖物点击回调接口。
	 * @param listener 标记覆盖物点击回调接口
	 */
	public void setOnMarkerClickListener(OnMarkerClickListener listener) {
		mOnMarkerClickListener = listener;
		if (listener != null) {
			mKMap.setOnMarkerClickListener(new KMap.OnMarkerClickListener() {
				
				@Override
				public boolean onMarkerClick(KMarker marker) {
					if(mOnMarkerClickListener != null && marker != null) {
						mOnMarkerClickListener.onMarkerClick(marker.getParent());
						return true;
					}
					return false;
				}
			});
		} else {
			mKMap.setOnMarkerClickListener(null);
		}
	}

	/**
	 * 设置标记覆盖物拖动回调接口。
	 * @param listener 标记覆盖物拖动回调接口
	 */
	public void setOnMarkerDragListener(OnMarkerDragListener listener) {
		mOnMarkerDragListener = listener;
		if (listener != null) {
			mKMap.setOnMapMarkerDragListener(new KMap.OnMarkerDragListener() {
				
				@Override
				public void onMarkerDragStart(KMarker marker) {
					if(mOnMarkerDragListener != null && marker != null) {
						mOnMarkerDragListener.onMarkerDragStart(marker.getParent());
					}
				}
				
				@Override
				public void onMarkerDragEnd(KMarker marker) {
				if(mOnMarkerDragListener != null && marker != null) {
					mOnMarkerDragListener.onMarkerDragEnd(marker.getParent());
				}
			}
				
				@Override
				public void onMarkerDrag(KMarker marker) {
					if(mOnMarkerDragListener != null && marker != null) {
						mOnMarkerDragListener.onMarkerDrag(marker.getParent());
					}
				}
			});
		} else {
			mKMap.setOnMapMarkerDragListener(null);
		}
	}

	/**
	 * 设置信息窗口创建回调接口。
	 * @param adapter 信息窗口创建回调接口
	 */
	public void setInfoWindowAdapter(InfoWindowAdapter adapter) {
		mInfoWindowAdapter = adapter;
		if (adapter != null) {
			mKMap.setInfoWindowAdapter(new KMap.InfoWindowAdapter() {
				
				@Override
				public View getInfoWindow(KMarker marker) {
					if(mInfoWindowAdapter != null && marker != null) {
						return mInfoWindowAdapter.getInfoWindow(marker.getParent());
					}
					return null;
				}
				
				@Override
				public View getInfoContents(KMarker marker) {
					return null;
				}
			});
		} else {
			mKMap.setInfoWindowAdapter(null);
		}
	}

	/**
	 * 设置信息窗口点击回调接口。
	 * @param listener 信息窗口点击回调接口
	 */
	public void setOnInfoWindowClickListener(OnInfoWindowClickListener listener) {
		mOnInfoWindowClickListener = listener;
		if (listener != null) {
			mKMap.setOnInfoWindowClickListener(new KMap.OnInfoWindowClickListener() {
				
				@Override
				public void onInfoWindowClick(KMarker marker) {
					if(mOnInfoWindowClickListener != null && marker != null) {
						mOnInfoWindowClickListener.onInfoWindowClick(marker.getParent());
					}
				}
			});
		} else {
			mKMap.setOnInfoWindowClickListener(null);
		}
	}

	/**
	 * 设置地图可视范围变化回调接口。
	 * @param listener 地图可视范围变化回调接口
	 */
	public void setOnCameraChangeListener(OnCameraChangeListener listener) {
		mOnCameraChangeListener = listener;
		if (listener != null) {
			mKMap.setOnCameraChangeListener(new KMap.OnCameraChangeListener() {
				
				@Override
				public void onCameraChangeFinish(KCameraPosition pos) {
					if(mOnCameraChangeListener != null) {
						mOnCameraChangeListener.onCameraChangeFinish(pos.toCameraPosition());
					}
				}
				
				@Override
				public void onCameraChange(KCameraPosition pos) {
					if(mOnCameraChangeListener != null) {
						mOnCameraChangeListener.onCameraChange(pos.toCameraPosition());
					}
				}
			});
		} else {
			mKMap.setOnCameraChangeListener(null);
		}
	}
	
	/**
	 * 发起地图截屏请求
	 * @param listener 地图截屏监听接口
	 */
	public void getMapScreenShot(OnMapScreenShotListener listener) {
	    mKMap.getMapScreenShot(listener);
	}

	/**
	 * 添加弧形覆盖物到地图上。
	 * @param options 弧形覆盖物选项对象
	 * @return 弧形覆盖物对象
	 */
	public Arc addArc(ArcOptions options) {
		Arc arc = null;
		KArc kArc = mKMap.addArc(new KArcOptions(options));
		if (kArc != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.Arc").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					arc = (Arc) cons[0].newInstance();
					kArc.setParent(arc);
					KOverlayManager.getInstance().addOverlay(arc.getId(), kArc, arc, mOnOverlayListener);
					overlays.add(arc);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return arc;
	}

	/**
	 * 添加圆形覆盖物到地图上。
	 * @param options 圆形覆盖物选项对象
	 * @return 圆形覆盖物对象
	 */
	public Circle addCircle(CircleOptions options) {
		Circle circle = null;
		KCircle kCircle = mKMap.addCircle(new KCircleOptions(options));
		if (kCircle != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.Circle").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					circle = (Circle) cons[0].newInstance();
					kCircle.setParent(circle);
					KOverlayManager.getInstance().addOverlay(circle.getId(), kCircle, circle, mOnOverlayListener);
					overlays.add(circle);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return circle;
	}

	/**
	 * 添加段线覆盖物到地图上。
	 * @param options 段线覆盖物选项对象
	 * @return 段线覆盖物对象
	 */
	public final Polyline addPolyline(PolylineOptions options) {
		Polyline polyline = null;
		KPolyline kPolyline = mKMap.addPolyline(new KPolylineOptions(options));
		if (kPolyline != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.Polyline").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					polyline = (Polyline) cons[0].newInstance();
					kPolyline.setParent(polyline);
					KOverlayManager.getInstance().addOverlay(polyline.getId(), kPolyline, polyline, mOnOverlayListener);
					overlays.add(polyline);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return polyline;
	}

	/**
	 * 添加多边形覆盖物到地图上。
	 * @param options 多边形覆盖物选项对象
	 * @return 多边形覆盖物对象
	 */
	public final Polygon addPolygon(PolygonOptions options) {
		Polygon polygon = null;
		KPolygon kPolygon = mKMap.addPolygon(new KPolygonOptions(options));
		if (kPolygon != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.Polygon").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					polygon = (Polygon) cons[0].newInstance();
					kPolygon.setParent(polygon);
					KOverlayManager.getInstance().addOverlay(polygon.getId(), kPolygon, polygon, mOnOverlayListener);
					overlays.add(polygon);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return polygon;
	}

	/**
	 * 添加图片覆盖物到到地图上。
	 * @param options 图片覆盖物选项对象
	 * @return 图片覆盖物对象
	 */
	public GroundOverlay addGroundOverlay(GroundOverlayOptions options) {
		GroundOverlay groundOverlay = null;
		KGroundOverlay kGroundOverlay = mKMap.addGroundOverlay(new KGroundOverlayOptions(options));
		if (kGroundOverlay != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.GroundOverlay").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					groundOverlay = (GroundOverlay) cons[0].newInstance();
					kGroundOverlay.setParent(groundOverlay);
					KOverlayManager.getInstance().addOverlay(groundOverlay.getId(), kGroundOverlay, groundOverlay, mOnOverlayListener);
					overlays.add(groundOverlay);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return groundOverlay;
	}

	/**
	 * 添加标记覆盖物到地图上。
	 * @param options 标记覆盖物选项对象
	 * @return 标记覆盖物对象
	 */
	public final Marker addMarker(MarkerOptions options) {
		Marker marker = null;
		KMarker kMarker = mKMap.addMarker(new KMarkerOptions(options));
		if (kMarker != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.Marker").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					marker = (Marker) cons[0].newInstance();
					kMarker.setParent(marker);
					KOverlayManager.getInstance().addOverlay(marker.getId(), kMarker, marker, mOnOverlayListener);
					overlays.add(marker);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return marker;
	}

	/**
	 * 添加多个标记覆盖物到地图上。
	 * @param options 多个标记覆盖物选项对象
	 * @return 多个标记覆盖物对象
	 */
	public final List<Marker> addMarkers(List<MarkerOptions> options) {
		ArrayList<Marker> arr = new ArrayList<Marker>();
		for (MarkerOptions option : options) {
			arr.add(addMarker(option));
		}
		return arr;
	}

	/**
	 * 添加箭头覆盖物对象到地图上。
	 * @param options 箭头覆盖物选项对象
	 * @return 箭头覆盖物对象
	 */
	public final NavigateArrow addNavigateArrow(NavigateArrowOptions options) {
		NavigateArrow arrow = null;
		KNavigateArrow kArrow = mKMap.addNavigateArrow(new KNavigateArrowOptions(options));
		if (kArrow != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.NavigateArrow").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					arrow = (NavigateArrow) cons[0].newInstance();
					KOverlayManager.getInstance().addOverlay(arrow.getId(), kArrow, arrow, mOnOverlayListener);
					overlays.add(arrow);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return arrow;
	}

	/**
	 * 添加栅格图层到地图上。
	 * @param options 栅格图层选项对象
	 * @return 栅格图层对象
	 */
	public final TileOverlay addTileOverlay(TileOverlayOptions options) {
		TileOverlay tileOverlay = null;
		KTileOverlay kTileOverlay = mKMap.addTileOverlay(new KTileOverlayOptions(options));
		if (kTileOverlay != null) {
			try {
				Constructor<?> cons[] = Class.forName("com.cmmap.api.maps.model.TileOverlay").getDeclaredConstructors();
				if (cons.length > 0) {
					cons[0].setAccessible(true);
					tileOverlay = (TileOverlay) cons[0].newInstance();
					KOverlayManager.getInstance().addOverlay(tileOverlay.getId(), kTileOverlay, tileOverlay, mOnOverlayListener);
					KOverlayManager.getInstance().addTileProvider(kTileOverlay);
					overlays.add(tileOverlay);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return tileOverlay;
	}

	/**
	 * 从地图上删除所有的覆盖物。
	 */
	public void clear() {
		for(int i = overlays.size() - 1; i >= 0 ; i--) {
			Object obj = overlays.get(i);
			if (obj instanceof Arc) {
				Arc arc = (Arc) obj;
				arc.remove();
			} else if (obj instanceof Circle) {
				Circle circle = (Circle) obj;
				circle.remove();
			} else if (obj instanceof Polyline) {
				Polyline polyline = (Polyline) obj;
				polyline.remove();
			} else if (obj instanceof Polygon) {
				Polygon polygon = (Polygon) obj;
				polygon.remove();
			} else if (obj instanceof GroundOverlay) {
				GroundOverlay ground = (GroundOverlay) obj;
				ground.remove();
			} else if (obj instanceof Marker) {
				Marker marker = (Marker) obj;
				marker.remove();
			} else if (obj instanceof NavigateArrow) {
				NavigateArrow arrow = (NavigateArrow) obj;
				arrow.remove();
			} else if (obj instanceof TileOverlay) {
				TileOverlay tileOverlay = (TileOverlay) obj;
				tileOverlay.remove();
			}
		}
		overlays.clear();
		mKMap.clear();
	}

	/**
	 * 返回地图用户界面设置对象。
	 * <br>
	 * 地图用户界面设置对象，可以允许用户控制地图自带控件的显示以及手势操作使能。
	 * @return 地图用户界面设置对象
	 */
	public final UiSettings getUiSettings() {
		return mUiSettings;
	}

	/**
	 * 返回地图坐标映射对象。
	 * <br>
	 * 用户通过这个对象可以在屏幕坐标与经纬度坐标之间进行转换。
	 * @return 地图坐标映射对象。
	 */
	public Projection getProjection() {
		return mProjection;
	}

	/**
	 * 返回地图显示类型。
	 * @return 地图显示类型，参见：
	 *         <br>{@link #MAP_TYPE_NORMAL}
	 *         <br>{@link #MAP_TYPE_NIGHT}
	 *         <br>{@link #MAP_TYPE_SATELLITE}
	 *         <br>{@link #MAP_TYPE_NAVI}
	 *         <br>{@link #MAP_TYPE_BUS}
     *         <br>{@link #MAP_TYPE_RP}
	 */
	public final int getMapType() {
		return mKMap.getMapType();
	}

	/**
	 * 设置地图显示的类型(建议在onMapLoaded之后调用)。
	 * @param type 地图显示类型，参见：
	 *         <br>{@link #MAP_TYPE_NORMAL}
	 *         <br>{@link #MAP_TYPE_NIGHT}
	 *         <br>{@link #MAP_TYPE_SATELLITE}
	 *         <br>{@link #MAP_TYPE_NAVI}
	 *         <br>{@link #MAP_TYPE_BUS}
	 *         <br>{@link #MAP_TYPE_RP}
	 */
	public void setMapType(int type) {
		mKMap.setMapType(type);
	}

	/**
	 * 返回地图最大缩放级别。
	 * @return 地图最大缩放级别
	 */
	public final float getMaxZoomLevel() {
		return mKMap.getMaxZoomLevel();
	}

	/**
	 * 返回地图最小缩放级别。
	 * @return 地图最小缩放级别
	 */
	public final float getMinZoomLevel() {
		return mKMap.getMinZoomLevel();
	}

	/**
	 * 获取当前地图缩放级别下，地图上单位像素点对应的长度。
	 * @return 当前缩放级别下，地图上单位像素点对应的长度，单位：米
	 */
	public float getScalePerPixel() {
		return mKMap.getScalePerPixel();
	}

	/**
	 * 返回当前地图可视范围。
	 * @return 当前地图可视范围
	 */
	public final CameraPosition getCameraPosition() {
		return mKMap.getCameraPosition().toCameraPosition();
	}

	/**
	 * 改变地图可视范围。
	 * <br>
	 * 这个方法为瞬间移动，没有移动过程。
	 * @param update 地图可视范围更新对象
	 */
	public final void moveCamera(CameraUpdate update) {
	    if (update == null) {
	        return;
	    }
		mKMap.moveCamera(update.mKCameraUpdate);
	}

	/**
	 * 以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 */
	public void animateCamera(CameraUpdate update) {
	    if (update == null) {
	        return;
	    }
		mKMap.animateCamera(update.mKCameraUpdate);
	}

	/**
	 * 以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 * @param cancelableCallback 如果动画正常完成，则会调用{@link CancelableCallback#onFinish()}方法，否则调用{@link CancelableCallback#onCancel()}
	 */
	public void animateCamera(CameraUpdate update, CancelableCallback cancelableCallback) {
	    if (update == null) {
	        return;
	    }
		animateCamera(update, 300, cancelableCallback);
	}

	/**
	 * 在指定的持续时间内，以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 * @param durationMs 动画持续时间，单位毫秒
	 * @param cancelableCallback 如果动画正常完成，则会调用onFinish()方法，否则调用onCancel()
	 */
	public void animateCamera(CameraUpdate update, long durationMs, CancelableCallback cancelableCallback) {
		if (update == null) {
	        return;
	    }
		mCancelableCallback = cancelableCallback;
		KMap.CancelableCallback kCancelableCallback = new KMap.CancelableCallback() {
			
			@Override
			public void onFinish() {
				if(mCancelableCallback != null) {
					mCancelableCallback.onFinish();
				}
			}
			
			@Override
			public void onCancel() {
				if(mCancelableCallback != null) {
					mCancelableCallback.onCancel();
				}
			}
		};
		mKMap.animateCamera(update.mKCameraUpdate, durationMs, kCancelableCallback);
	}

	/**
	 * 停止动画效果。
	 * <br>
	 * 该方法被调用后，地图可视范围改变立即停止并停留当前位置，该函数用于取消{@link #animateCamera}调用
	 */
	public void stopAnimation() {
		mKMap.stopAnimation();
	}

	/**
	 * 返回地图底图文字的z轴索引。
	 * @return 地图底图文字的z轴索引，默认：0
	 */
	public final int getMapTextZIndex() {
		return (int) mKMap.getMapTextZIndex();
	}

	/**
	 * 设置地图底图文字的z轴索引。
	 * @param zIndex 地图底图文字的z轴索引
	 */
	public void setMapTextZIndex(int zIndex) {
		mKMap.setMapTextZIndex(zIndex);
	}

	/**
	 * 设置屏幕上的某个点为地图视图旋转中心点，默认值为(0.5,0.5)，标识以地图视图中心点为地图视图旋转中心点。
	 * @param x 地图视图旋转中心点横向比例，范围为 (0,1)，默认：0.5
	 * @param y 地图视图旋转中心点横向比例，范围为 (0,1)，默认：0.5
	 */
	public void setPointToCenter(float x, float y) {
		mKMap.setPointToCenter(x, y);
	}

	/**
	 * 设置是否显示地图3D建筑物，默认：显示。
	 * @param enabled 是否显示地图3D建筑物
	 */
	public void showBuildings(boolean enabled) {
		mKMap.showBuildings(enabled);
	}
	
	/**
	 * 设置是否显示天空背景，默认：显示。
	 * @param enabled 是否显示天空背景
	 */
	public void showSkyBackground(boolean enabled) {
		mKMap.showSkyBackground(enabled);
	}
	
	/**
	 * 设置是否显示地图底图文字，默认：显示。 
	 * @param enabled 是否显示地图底图文字
	 */
	public void showMapText(boolean enabled) {
		mKMap.showMapText(enabled);
	}

	/**
	 * 返回是否显示实时交通。
	 * @return 是否显示实时交通，默认：否
	 */
	public final boolean isTrafficEnabled() {
		return mKMap.isTrafficEnabled();
	}

	/**
	 * 设置是否显示实时交通。
	 * @param enabled 是否显示实时交通
	 */
	public void setTrafficEnabled(boolean enabled) {
		mKMap.setTrafficEnabled(enabled);
	}
	
	/**
	 * 返回是否显示路况事件。
	 * @return 是否显示路况事件，默认：否
	 */
	public final boolean isTrafficEventEnabled() {
	    return mKMap.isTrafficEventEnabled();
	}
	
	/**
	 * 设置是否显示路况事件
	 * @param enabled 是否显示路况事件
	 */
	public void setTrafficEventEnabled(boolean enabled) {
	    mKMap.setTrafficEventEnabled(enabled);
	}
	
	/**
	 * 返回实时交通路况拥堵对应的颜色属性。
	 * @return 实时交通路况拥堵对应的颜色属性，参见{@link MyTrafficStyle}
	 */
	public MyTrafficStyle getMyTrafficStyle() {
		return mKMap.getMyTrafficStyle();
	}

	/**
	 * 设置实时交通自定义的路况拥堵颜色属性。
	 * @param style 实时交通自定义的路况拥堵颜色属性，参见{@link MyTrafficStyle}
	 */
	public void setMyTrafficStyle(MyTrafficStyle style) {
		mKMap.setMyTrafficStyle(style);
	}

	/**
	 * 删除地图存储目录下的手机地图缓存, 没有回调。
	 */
	public void removecache() {
		mKMap.removeCache();
	}
	
	/**
	 * 设置文本放大比例
	 * @param rate 文本放大比例
	 */
	public void SetTextSizeZoomRate(float rate) {
		if (rate < 0) {
			return;
		}
		mKMap.SetTextSizeZoomRate(rate);
	}
	
	/**
	 * 返回SDK版本号。
	 * 这是个静态方法，可以在一切地图接口前调用。
	 * @param context 应用程序的上下文
	 * @return SDK版本号
	 */
	public static String getSDKVersion(Context context) {
		return DataOperator.getSDKVersion(context);
	}
	
	/**
     * 设置卫片数据url
     * @param strTileUrl 卫片数据url
     * @param xOrder 参数x的次序，取值范围0,1,2
     * @param yOrder 参数y的次序，取值范围0,1,2
     * @param zOrder 参数z的次序，取值范围0,1,2
     */
    public void setTileUrl(String strTileUrl, int xOrder, int yOrder, int zOrder)
    {
    	mKMap.setTileUrl(strTileUrl, xOrder, yOrder, zOrder);
    }
    
    /**
	 * 初始化配置项,可在map对象创建前提前初始化
	 * @param context
	 */
    public static void initConfig(Context context)
    {
    	KMap.initConfig(context);
    }
    
    /**
     * 获取地图审图号（普通地图）
     * @return 普通矢量地图审图号
     */
    public String getMapContentApprovalNumber() {
        return mKMap.getMapContentApprovalNumber();
    }
}
