package com.cmmap.api.maps.model;

import com.cmmap.internal.maps.model.KArc;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 弧形覆盖物类。
 * <p>它有以下属性：</p>
 * <ul>
 * <li>边框宽度<br>这个值设置后不会受到可视区域缩放级别的影响。</li>
 * <li>边框颜色<br>边框颜色是一个ARGB格式，ARGB请参考{@link android.graphics.Color}。默认为黑色。</li> 
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class Arc {
	private Integer mId = KOverlayManager.assignId();
	private KArc mKArc = null;
	private static final int POINT_NUM = 3;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Arc() {
	}
	
	private boolean findKArc() {
		if (mKArc == null) {
			mKArc = (KArc) KOverlayManager.getInstance().findOverlay(getId());
		}
		return (mKArc != null);
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth() {
		if (findKArc()) {
			return mKArc.getStrokeWidth();
		}
		return 0;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，必须大于等于0，如果为0，则不会绘制边框
	 */
	public void setStrokeWidth(float width) {
		if (findKArc()) {
			mKArc.setStrokeWidth(width);
		}
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色。
	 */
	public int getStrokeColor() {
		if (findKArc()) {
			return mKArc.getStrokeColor();
		}
		return 0;
	}

	/**
	 * 设置边框颜色。
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setStrokeColor(int color) {
		if (findKArc()) {
			mKArc.setStrokeColor(color);
		}
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findKArc()) {
			return mKArc.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findKArc()) {
			mKArc.setDraggable(enable);
		}
	}

	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findKArc()) {
			return mKArc.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findKArc()) {
			mKArc.setLevel(level);
		}
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findKArc()) {
			return mKArc.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findKArc()) {
			mKArc.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findKArc()) {
			return mKArc.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findKArc()) {
			mKArc.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findKArc()) {
			mKArc.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mKArc = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override 
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof Arc)) {
			return false;
		}
		
		Arc arc = (Arc) object;
		return arc.getId().equals(getId());
	}
}
