package com.cmmap.api.maps.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.view.View;
import android.view.View.MeasureSpec;

import com.cmmap.internal.driver.base.GlobalSettings;

/**
 * 图片描述符创建类。
 */
public class BitmapDescriptorFactory {

	/**
	 * 创建默认的marker 图标BitmapDescriptor 对象
	 * @return
	 */
	static BitmapDescriptor defaultMarker(){
		return fromAsset("cmmap/mapres/default_marker.png");
		
	}
	
	/**
	 * 根据传入的Bitmap对象创建{@link BitmapDescriptor}对象。
	 * @param image Bitmap对象
	 * @return {@link BitmapDescriptor}对象
	 */
	public static BitmapDescriptor fromBitmap(Bitmap image) {
		BitmapDescriptor descriptor = new BitmapDescriptor();
		descriptor.setBitmap(Bitmap.createBitmap(image));
		return descriptor;
	}

	/**
	 * 根据传入的view，创建{@link BitmapDescriptor}对象。
	 * @param view view对象
	 * @return {@link BitmapDescriptor}对象
	 */
	public static BitmapDescriptor fromView(View view) {
		if (view == null) {
			return null;
		}
		
		view.measure(MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED), MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED));
        view.layout(0, 0, view.getMeasuredWidth(), view.getMeasuredHeight());
        view.buildDrawingCache();
        Bitmap viewBitmap = view.getDrawingCache();
        BitmapDescriptor descriptor = null;
        if (viewBitmap != null && !viewBitmap.isRecycled()) {
        	Bitmap bitmap = viewBitmap.copy(viewBitmap.getConfig(), false);
        	descriptor = new BitmapDescriptor();
            descriptor.setBitmap(bitmap);
		}
		
		return descriptor;
	}

	/**
	 * 根据传入的asset目录内的图片文件名称创建{@link BitmapDescriptor}对象。
	 * @param assetName asset目录内的图片文件名称
	 * @return {@link BitmapDescriptor}对象
	 */
	/**
	 * @param assetName
	 * @return
	 */
	public static BitmapDescriptor fromAsset(String assetName) {
		BitmapDescriptor descriptor = new BitmapDescriptor();
		InputStream is_bitmap;
		Context context = GlobalSettings.getInstance().getContext();
		if (context != null) {
			try {
				is_bitmap = context.getAssets().open(assetName);
				if(is_bitmap != null){
					Bitmap _bitmap = BitmapFactory.decodeStream(is_bitmap);
					descriptor.setBitmap(_bitmap);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		return descriptor;
	}

	/**
	 * 根据传入的图片文件的资源ID，创建{@link BitmapDescriptor}对象。
	 * @param resourceId 图片文件的资源ID
	 * @return {@link BitmapDescriptor}对象
	 */
	public static BitmapDescriptor fromResource(int resourceId) {
		BitmapDescriptor descriptor = new BitmapDescriptor();
		Context context = GlobalSettings.getInstance().getContext();
		if (context != null) {
			Bitmap bitmap = BitmapFactory.decodeResource(context.getResources(), resourceId);
			if (bitmap != null) {
				descriptor.setBitmap(bitmap);				
			}
		}
		return descriptor;
	}

	/**
	 * 根据传入的在手机存储器里的图片文件名，创建的{@link BitmapDescriptor}对象。
	 * @param fileName 手机存储器里的图片文件名，应该包括路径
	 * @return {@link BitmapDescriptor}
	 */
	public static BitmapDescriptor fromFile(String fileName) {
		BitmapDescriptor descriptor = new BitmapDescriptor();
		//TODO 方法有问题，文件变大
		File file = new File(fileName);
		if (!file.exists()) {
			return descriptor;
		}
		InputStream is = null;
		try {
			is = new FileInputStream(file);
			if (is != null) {
				Bitmap _bitmap = BitmapFactory.decodeStream(is);
				descriptor.setBitmap(_bitmap);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} finally {
			try {
				if (is != null) {
					is.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return descriptor;
	}
}
