package com.cmmap.api.maps.model;

import android.os.Parcel;
import android.os.Parcelable;

import com.cmmap.internal.maps.KCameraUpdate;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图可视范围类。
 * <p>地图可视范围类描述了地图显示相关的属性，包括：地图中心点坐标、地图旋转角度、地图倾斜角度、地图缩放级别。</p>
 */
public final class CameraPosition implements Parcelable {
	
	KCameraPosition kCameraPosition;
	/** 地图中心点坐标 */
	public final LatLng target; 
	/** 地图旋转角度，以角度为单位，从正北向逆时针方向计算，范围：[0, 360) */
	public final float bearing; 
	/** 地图倾斜角度，以角度为单位 ，范围：[0, 45] */
	public final float tilt; 
	/** 地图缩放级别 */
	public final float zoom; 

	/**
	 * 构造函数。
	 * @param target 地图中心点坐标
	 * @param zoom 地图缩放级别
	 * @param tilt 地图倾斜角度
	 * @param bearing 地图旋转角度
	 */
	public CameraPosition(LatLng target, float zoom, float tilt, float bearing) {
		if (target == null)
			target = new LatLng(0.0, 0.0);
		this.target = target;
		this.zoom = zoom;
		this.tilt = tilt;
		this.bearing = bearing;
		kCameraPosition = new KCameraPosition(new KLatLng(target.latitude,
				target.longitude), zoom, tilt, bearing);
	}



	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Float.floatToIntBits(bearing);
		result = prime * result
				+ ((kCameraPosition == null) ? 0 : kCameraPosition.hashCode());
		result = prime * result + ((target == null) ? 0 : target.hashCode());
		result = prime * result + Float.floatToIntBits(tilt);
		result = prime * result + Float.floatToIntBits(zoom);
		return result;
	}



	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CameraPosition other = (CameraPosition) obj;
		if (Float.floatToIntBits(bearing) != Float
				.floatToIntBits(other.bearing))
			return false;
		if (kCameraPosition == null) {
			if (other.kCameraPosition != null)
				return false;
		} else if (!kCameraPosition.equals(other.kCameraPosition))
			return false;
		if (target == null) {
			if (other.target != null)
				return false;
		} else if (!target.equals(other.target))
			return false;
		if (Float.floatToIntBits(tilt) != Float.floatToIntBits(other.tilt))
			return false;
		if (Float.floatToIntBits(zoom) != Float.floatToIntBits(other.zoom))
			return false;
		return true;
	}



	@Override
	public String toString() {
		return String.format("{target=%s,bearing=%s,tilt=%s,zoom=%s}", target, bearing, tilt, zoom);
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeValue(target);
		dest.writeFloat(bearing);
		dest.writeFloat(tilt);
		dest.writeFloat(zoom);
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<CameraPosition> CREATOR = new Parcelable.Creator<CameraPosition>() {
		@Override
		public CameraPosition createFromParcel(Parcel source) {
			LatLng latLng = (LatLng) source.readValue(LatLng.class.getClassLoader());
			float bearing = source.readFloat();
			float tilt = source.readFloat();
			float zoom = source.readFloat();
			CameraPosition c = new CameraPosition(latLng, zoom, tilt, bearing);
			return c;
		}

		@Override
		public CameraPosition[] newArray(int size) {
			return new CameraPosition[size];
		}
	};
}
