package com.cmmap.api.maps.model;

import com.cmmap.internal.maps.model.KCircle;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 圆形覆盖物类。
 * <p>它有有以下属性：</p>
 * <ul>
 * <li>圆心 <br>圆形圆心的经纬度。</li>
 * <li>半径<br>单位：米，这个值应该大于等于0。</li>
 * <li>边框宽度<br>这个值设置后不会受到地图域缩放级别的影响，单位：像素，默认为10。</li>
 * <li>边框颜色<br>边框的颜色，ARGB格式，参见{@link android.graphics.Color}，默认为黑色。</li>
 * <li>填充颜色<br>圆形填充的颜色，ARGB格式参见{@link android.graphics.Color}，默认为透明。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public final class Circle {
	Integer mId = KOverlayManager.assignId();
	KCircle mKCircle = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Circle() {
	}
	
	private boolean findCircle() {
		if (mKCircle == null) {
			mKCircle = (KCircle) KOverlayManager.getInstance().findOverlay(getId());
		}
		return (mKCircle != null);
	}

	/**
	 * 返回圆心坐标。
	 * @return 圆心坐标
	 */
	public LatLng getCenter() {
		if (findCircle()) {
			return mKCircle.getCenter().toLatLng();
		}
		return null;
	}

	/**
	 * 设置圆心坐标，参数不能为空
	 * @param center 圆心坐标
	 */
	public void setCenter(LatLng center) {
		if (findCircle()) {
			mKCircle.setCenter(new KLatLng(center.latitude, center.longitude));
		}
	}

	/**
	 * 返回圆形半径。
	 * @return 圆形半径，单位：米
	 */
	public double getRadius() {
		if (findCircle()) {
			mKCircle.getRadius();
		}
		return 0;
	}

	/**
	 * 圆形半径。
	 * @param radius 圆形半径，单位：米，半径必须大于等于0
	 */
	public void setRadius(double radius) {
		if (findCircle()) {
			mKCircle.setRadius(radius);
		}
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */ 
	public float getStrokeWidth() {
		if (findCircle()) {
			return mKCircle.getStrokeWidth();
		}
		return 0;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，参数必须大于等于0，如果为0则不会绘制边框
	 */
	public void setStrokeWidth(float width) {
		if (findCircle()) {
			mKCircle.setStrokeWidth(width);
		}
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		if (findCircle()) {
			mKCircle.getStrokeColor();
		}
		return 0;
	}

	/**
	 * 设置边框颜色
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setStrokeColor(int color) {
		if (findCircle()) {
			mKCircle.setStrokeColor(color);
		}
	}

	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor() {
		if (findCircle()) {
			mKCircle.getFillColor();
		}
		return 0;
	}

	/**
	 * 设置填充颜色。
	 * @param color 填充颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setFillColor(int color) {
		if (findCircle()) {
			mKCircle.setFillColor(color);
		}
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findCircle()) {
			return mKCircle.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findCircle()) {
			mKCircle.setDraggable(enable);
		}
	}

	/**
	 * 判断圆形是否包含传入的经纬度点。
	 * @param latLng 经纬度点
	 * @return 是否包含 
	 */
	public boolean contains(LatLng latLng) {
		if (findCircle()) {
			return mKCircle.contains(new KLatLng(latLng.latitude, latLng.longitude));
		}
		return false;
	}

	
	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findCircle()) {
			return mKCircle.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findCircle()) {
			mKCircle.setLevel(level);
		}
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findCircle()) {
			return mKCircle.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findCircle()) {
			mKCircle.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findCircle()) {
			return mKCircle.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findCircle()) {
			mKCircle.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findCircle()) {
			mKCircle.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mKCircle = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override 
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof Circle)) {
			return false;
		}
		
		Circle circle = (Circle) object;
		return circle.getId().equals(getId());
	}
}
