package com.cmmap.api.maps.model;

import android.graphics.Color;

/**
 * 圆形覆盖物选项类。
 */
public final class CircleOptions{
	LatLng m_centerPoint = new LatLng(0, 0);
	double m_radius = 0;
	float m_strokeWidth = 10;
	int m_strokeColor = Color.BLACK;
	int m_fillColor = Color.BLACK;
	boolean m_draggable = false;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	
	/**
	 * 返回圆心坐标
	 * @return 圆点坐标
	 */
	public LatLng getCenter() {
		return m_centerPoint;
	}

	/**
	 * 设置圆心坐标。
	 * @param point 圆心坐标，参数不能为空
	 * @return 该对象本身
	 */
	public CircleOptions center(LatLng point) {
		m_centerPoint = new LatLng(point.latitude, point.longitude);
		return this;
	}

	/**
	 * 返回圆形半径。
	 * @return 圆形半径，单位：米
	 */
	public double getRadius() {
		return m_radius;
	}

	/**
	 * 设置圆形半径。
	 * @param radius 圆形半径，单位：米，半径必须大于等于0
	 * @return 该对象本身
	 */
	public CircleOptions radius(double radius) {
		m_radius = radius;
		return this;
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位像素
	 */
	public float getStrokeWidth() {
		return m_strokeWidth;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位像素，参数必须大于等于0，如果为0则不会绘制边框，默认：10。
	 * @return 该对象本身
	 */
	public CircleOptions strokeWidth(float width) {
		m_strokeWidth = width;
		return this;
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		return m_strokeColor;
	}

	/**
	 * 设置边框颜色。
	 * @param color 设置边框颜色 ，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public CircleOptions strokeColor(int color) {
		m_strokeColor = color;
		return this;

	}

	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor() {
		return m_fillColor;
	}
	
	/**
	 * 设置填充颜色。
	 * @param color 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public CircleOptions fillColor(int color) {
		m_fillColor = color;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public CircleOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}

	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public CircleOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public CircleOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public CircleOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
