package com.cmmap.api.maps.model;

import com.cmmap.internal.maps.model.KGroundOverlay;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 图片覆盖物类。
 * <p>图片覆盖物层是一张图片以合适的大小贴在地图上，包含如下属性：</p>
 * <ul>
 * <li>图片显示范围<br>定义了图片显示的具体经纬度范围。</li>
 * <li>图片<br>给图片层的贴图。在添加图片层之前，如果没有设置图片，会报IllegalArgumentException 异常。</li>
 * <li>图片透明度<br>定义了图片的透明度，范围[0, 1.0]，0为不透明，1为全透明。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class GroundOverlay {
	Integer mId = KOverlayManager.assignId();
	KGroundOverlay mKGroundOverlay = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	GroundOverlay() {
	}
	
	private boolean findGroundOverlay() {
		if (mKGroundOverlay == null) {
			mKGroundOverlay = (KGroundOverlay) KOverlayManager.getInstance().findOverlay(getId());
		}
		return (mKGroundOverlay != null);
	}

	/**
	 * 返回图片显示的的矩形区域 。
	 * @return 图片显示的的矩形区域 
	 */
	public LatLngBounds getBounds() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.getBounds();
		}
		return null;
	}

	/**
	 * 根据矩形区域设置图片显示的矩形区域。
	 * @param latLngBounds 矩形区域
	 */
	public void setBounds(LatLngBounds latLngBounds) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setPositionFromBounds(latLngBounds);
		}
	}
	
	/**
	 * 返回图片对象。
	 * @return 图片对象
	 */
	public BitmapDescriptor getImage() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.getImage();
		}
		return null;
	}

	/**
	 * 设置图片对象。
	 * @param image 图片对象
	 */
	public void setImage(BitmapDescriptor image) {
		if (findGroundOverlay() && image != null) {
			mKGroundOverlay.setImage(image);
		}
	}

	/**
	 * 返回绘制图片时的的透明度。
	 * @return 绘制图片时的的透明度，默认：0
	 */
	public float getTransparency() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.getTransparency();
		}
		return 0;
	}

	/**
	 * 设置绘制图片时的的透明度。
	 * @param transparency 绘制图片时的的透明度，范围为[0,1]，0为不透明，1为全透明
	 */
	public void setTransparency(float transparency) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setTransparency(transparency);
		}
		
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setDraggable(enable);
		}
	}

	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setLevel(level);
		}
	}

	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findGroundOverlay()) {
			return mKGroundOverlay.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findGroundOverlay()) {
			mKGroundOverlay.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findGroundOverlay()) {
			mKGroundOverlay.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mKGroundOverlay = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if(object == null 
				|| !(object instanceof GroundOverlay)) {
			return false;
		}
		
		GroundOverlay groundOverlay = (GroundOverlay) object;
		return groundOverlay.getId().equals(getId());
	}
}
