package com.cmmap.api.maps.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 经纬度矩形类。
 */
public final class LatLngBounds implements Parcelable {
	/** 东北方向经纬度 */
	public final LatLng northeast;
	/** 西南方向经纬度 */
	public final LatLng southwest;

	/**
	 * 构造函数。
	 * @param northeast 东北方向经纬度
	 * @param southwest 西南方向经纬度
	 */
	public LatLngBounds(LatLng northeast, LatLng southwest) {
		this.northeast = northeast;
		this.southwest = southwest;
	}

	/**
	 * 判断矩形区域是否包含传入的经纬度点。
	 * @param point 经纬度点
	 * @return 是否包含
	 */
	private boolean contains(LatLng point) {
		if (point == null) {
			return false;
		}
		if (southwest == null || northeast == null) {
			return false;
		}
		if (point.latitude >= southwest.latitude && point.latitude <= northeast.latitude) {
			return containsLon(point);
		}
		return false;
	}
	
	private boolean containsLon(LatLng point) {
		if(point == null){
			return false;
		}
		if (southwest == null || northeast == null) {
			return false;
		}
		if(southwest.longitude < northeast.longitude
				&&point.longitude >= southwest.longitude  && point.longitude<= northeast.longitude)
		{
			return true;
		}
		else if(southwest.longitude > northeast.longitude
				&&point.longitude <= southwest.longitude && point.longitude>= northeast.longitude)
		{
			return true;
		}
		return false;
	}

	/**
	 * 判断传入的矩形区域是否在当前矩形区域之外。
	 * @param curBounds 
	 * @return
	 */
	private boolean outSide(LatLngBounds curBounds){
		if (curBounds == null || curBounds.southwest == null
				|| curBounds.northeast == null) {
			return false;
		}
		if (southwest == null || northeast == null) {
			return false;
		}
		if(southwest.longitude > curBounds.northeast.longitude
				|| northeast.longitude < curBounds.southwest.longitude
				|| southwest.latitude < curBounds.northeast.latitude
				|| northeast.longitude > curBounds.southwest.latitude
				)
		{
			return true;
		}
		//传入的bounds在当前bounds之外
		return false;
	}
	/**
	 * 返回一个新的矩形区域。
	 * @param point 扩展点的经纬度对象
	 * @return 新的{@link LatLngBounds}对象
	 */ 
	public LatLngBounds including(LatLng point) {
		return include(point);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((northeast == null) ? 0 : northeast.hashCode());
		result = prime * result
				+ ((southwest == null) ? 0 : southwest.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		LatLngBounds other = (LatLngBounds) obj;
		if (northeast == null) {
			if (other.northeast != null)
				return false;
		} else if (!northeast.equals(other.northeast))
			return false;
		if (southwest == null) {
			if (other.southwest != null)
				return false;
		} else if (!southwest.equals(other.southwest))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return String.format("{northeast=%s,southwest=%s}", northeast, southwest);
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public void writeToParcel(Parcel out, int flags) {
		out.writeValue(northeast);
		out.writeValue(southwest);
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<LatLngBounds> CREATOR = new Parcelable.Creator<LatLngBounds>() {
		@Override
		public LatLngBounds createFromParcel(Parcel source) {
			LatLng northeast = (LatLng) source.readValue(LatLng.class.getClassLoader());
			LatLng southwest = (LatLng) source.readValue(LatLng.class.getClassLoader());
			LatLngBounds latLngBounds = new LatLngBounds(northeast, southwest);
			return latLngBounds;
		}

		@Override
		public LatLngBounds[] newArray(int size) {
			return new LatLngBounds[size];
		}
	};

	/**
	 * 返回一个新的矩形区域
	 * @param point 扩展点的经纬度对象
	 * @return 一个根据传入的经纬度对原有区域进行最小的扩展的新矩形区域
	 */
	private LatLngBounds include(LatLng point){
		LatLng sLatLng = southwest;
		LatLng nLatLng = northeast;
		if(contains(point))
		{
			return this;
		}
		if (sLatLng == null) {
			sLatLng = new LatLng(0.0, 0.0);
		}
		if (nLatLng == null) {
			nLatLng = new LatLng(0.0, 0.0);
		}

		if(point.latitude < sLatLng.latitude)
		{
			sLatLng = new LatLng(point.latitude, sLatLng.longitude);
		}
		if(point.latitude > nLatLng.latitude)
		{
			nLatLng = new LatLng(point.latitude, nLatLng.longitude);
		}
		//经度支持-179度~180度
		if (containsLon(point)) {
			// 经度在范围内不扩展
		}
		// 向东或向西进行最小的扩展，优先向东扩展
		else if (lngDiff(nLatLng.longitude, point.longitude) <= lngDiff(
				point.longitude, sLatLng.longitude)) {
			nLatLng = new LatLng(nLatLng.latitude, point.longitude);
		}
		else
		{
			sLatLng = new LatLng(sLatLng.latitude, point.longitude);
		}
		LatLngBounds retBounds = new LatLngBounds(nLatLng, sLatLng);
		return retBounds;
	}
	
	private double lngDiff(double lng1, double lng2){
		double diff = lng2 - lng1;
		if(diff < 0){
			diff = 360 + diff;
		}
		return diff;
	}
	
	/**
	 * LatLngBounds构造类
	 */
	public static final class Builder {
		private LatLngBounds bound = null;
		
		private void clearBound(){
			bound = null;
		}
		/**
		 * 创建矩形区域。
		 * @return 一个LatLngBounds对象
		 */
		public LatLngBounds build() {
			if(bound == null)
			{
				return new LatLngBounds(null, null);
			}
			else
			{
				return new LatLngBounds(bound.northeast, bound.southwest);
			}
		}

		/**
		 * 区域包含传入的坐标。
		 * @param point 将坐标点包含在区域中
		 * @return 加入新坐标的builder对象
		 */
		public LatLngBounds.Builder include(LatLng point)
		{
			if (bound == null) {
				bound = new LatLngBounds(point, point);
			} else {
				bound = bound.include(point);
			}
			return this;
		}
	}

	/**
	 * 构造一个{@link LatLngBounds.Builder}对象。
	 * @return 新创建的{@link LatLngBounds.Builder}对象
	 */
	public static LatLngBounds.Builder builder() {
		return new LatLngBounds.Builder();
	}
}
