package com.cmmap.api.maps.model;

import java.util.List;

import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 标记覆盖物类。
 * <p>标记覆盖物是在地图上绘制一个带图标的点。一个标记覆盖物有如下属性：</p>
 * <ul>
 * <li>锚点<br>图标摆放在地图上的基准点。默认情况下，锚点是从图片下沿的中间处。</li>
 * <li>位置<br>Marker通过经纬度的值来标注在地图上的。</li>
 * <li>标题<br>默认的信息窗口的显示标题，随时可以更改。</li>
 * <li>描述<br>默认的信息窗口的显示描述，随时可以更改。</li>
 * <li>图标<br>Marker显示的图标。如果未设置图标，API将使用默认的图标。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public final class Marker {
	Integer mId = KOverlayManager.assignId();
	KMarker mKMarker = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Marker() {
	}
	
	private boolean findMarker() {
		if (mKMarker == null) {
			mKMarker = (KMarker) KOverlayManager.getInstance().findOverlay(getId());
		}
		
		return (mKMarker != null);
	}

	/**
	 * 返回当前显示的经纬度点。
	 * @return 当前显示的经纬度点
	 */
	public LatLng getPosition() {
		if (findMarker()) {
			return mKMarker.getPosition().toLatLng();
		}
		return null;
	}

	/**
	 * 设置当前显示的经纬度点。 
	 * @param latlng 当前显示的经纬度点
	 */
	public void setPosition(LatLng latlng) {
		if (findMarker()) {
			mKMarker.stopAnimates();
			mKMarker.setPosition(new KLatLng(latlng.latitude, latlng.longitude));
		}
	}

	/**
	 * 设置Maker固定显示在屏幕上的像素坐标位置。
	 * <br>若用此方式，marker固定显示在屏幕上；如果想让标记随地图移动，可以使用{@link #setPosition} 改变。
	 * @param x 横向像素点，相对于地图视图的左上角
	 * @param y 纵向像素点，相对于地图视图的左上角
	 */
	public void setPositionByPixels(int x, int y) {
		if (findMarker()) {
			mKMarker.stopAnimates();
			mKMarker.setPositionByPixels(x, y);
		}
	}

	/**
	 * 返回图标列表，如果用户仅设置了一个图标，则返回一个图标。
	 * @return 图标列表
	 */
	public List<BitmapDescriptor> getIcons() {
		if (findMarker()) {
			return mKMarker.getIcons();
		}
		return null;
	}

	/**
	 * 设置单个图标。
	 * @param icon 图标
	 */
	public void setIcon(BitmapDescriptor icon) {
		if (findMarker()) {
			mKMarker.setIcon(icon);
		}
	}

	/**
	 * 设置多个图片，描点和大小以第一个图片为准，建议图片大小保持一致。
	 * @param icons 图片列表
	 */
	public void setIcons(List<BitmapDescriptor> icons) {
		if (findMarker()) {
			mKMarker.setIcons(icons);
		}
	}

	/**
	 * 返回图片交替刷新周期。
	 * @return 图片交替刷新周期，单位：ms，默认：50ms
	 */
	public int getPeriod() {
		if (findMarker()) {
			return mKMarker.getPeriod();
		}
		return 0;
	}

	/**
	 * 设置图片交替刷新周期，仅设置了多个图片才起作用。
	 * @param period 图片交替刷新周期
	 */
	public void setPeriod(int period) {
		if (findMarker()) {
			mKMarker.setPeriod(period);
		}
	}

	/**
	 * 返回锚点在水平范围的比例。
	 * @return 锚点在水平范围的比例
	 */
	public float getAnchorU() {
		if (findMarker()) {
			return mKMarker.getAnchorU();
		}
		return 0.5f;
	}

	/**
	 * 返回锚点垂直范围的比例。
	 * @return 锚点在垂直范围的比例
	 */
	public float getAnchorV() {
		if (findMarker()) {
			return mKMarker.getAnchorV();
		}
		return 1.0f;
	}

	/**
	 * 设置当前marker的锚点。 锚点是定位图标接触地图平面的点。图标的左上顶点为（0,0）点，右下点为（1,1）点。默认情况下，锚点为（0.5,1.0）。
	 * @param anchorU 锚点水平范围的比例
	 * @param anchorV 锚点垂直范围的比例
	 */
	public void setAnchor(float anchorU, float anchorV) {
		if (findMarker()) {
			mKMarker.setAnchor(anchorU, anchorV);
		}
	}

	/**
	 * 返回图片旋转角度。
	 * @return 图片旋转角度，从正北开始，逆时针计算
	 */
	public float getRotateAngle() {
		if (findMarker()) {
			return mKMarker.getRotateAngle();
		}
		return 0;
	}

	/**
	 * 设置图片旋转角度。
	 * @param angle 图片旋转角度，从正北开始，逆时针计算
	 */
	public void setRotateAngle(float angle) {
		if (findMarker()) {
			mKMarker.setRotateAngle(angle);
		}
	}

	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findMarker()) {
			return mKMarker.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findMarker()) {
			mKMarker.setDraggable(enable);
		}
	}
	
		/**
	 * 返回是否可点击。
	 * @return 是否可点击，默认：否
	 */
	public boolean isClickable() {
		if (findMarker()) {
			return mKMarker.isClickable();
		}
		return false;
	}

	
	/**
	 * 设置是否可点击。
	 * @param enable 是否可点击
	 */
	public void setClickable(boolean enable) {
		if (findMarker()) {
			mKMarker.setClickable(enable);
		}
	}

	/**
	 * 返回是否平贴地图显示。
	 * @return 是否平贴地图，默认：否
	 */
	public boolean isFlat() {
		if (findMarker()) {
			return mKMarker.isFlat();
		}
		return false;
	}

	/**
	 * 设置是否平贴地图显示。
	 * @param flat 是否平贴地图
	 */
	public void setFlat(boolean flat) {
		if (findMarker()) {
			mKMarker.setFlat(flat);
		}
	}

	/**
	 * 设置当前marker在最上面显示。
	 */
	public void setToTop() {
		if (findMarker()) {
			mKMarker.setToTop();
		}
	}

	/**
	 * 返回MMarker的附加对象。
	 * @return Marker的附加对象
	 */
	public Object getObject() {
		if (findMarker()) {
			return mKMarker.getObject();
		}
		return null;
	}
	
	/**
	 * 设置Marker的附加对象。
	 * <br>Marker不处理该属性，仅提供保存，供用户使用。
	 * @param object Marker的附加对象
	 */
	public void setObject(Object object) {
		if (findMarker()) {
			mKMarker.setObject(object);
		}
	}

	/**
	 * 返回标题信息。
	 * @return 标题信息
	 */
	public String getTitle() {
		if (findMarker()) {
			return mKMarker.getTitle();
		}
		return null;
	}

	/**
	 * 设置 标题信息。
	 * @param title 标题信息
	 */
	public void setTitle(String title) {
		if (findMarker()) {
			mKMarker.setTitle(title);
		}
	}

	/**
	 * 返回描述信息。
	 * @return 描述信息
	 */
	public String getSnippet() {
		if (findMarker()) {
			return mKMarker.getSnippet();
		}
		return null;
	}

	/**
	 * 设置 描述信息。
	 * @param snippet 描述信息
	 */
	public void setSnippet(String snippet) {
		if (findMarker()) {
			mKMarker.setSnippet(snippet);
		}
	}

	/**
	 * 返回当前Marker信息窗口是否正在显示。
	 * <br>此方法返回值时不会参考这个信息窗口是否在当前屏幕上真的可见。
	 * @return 当前Marker信息窗口是否正在显示，默认：隐藏
	 */
	public boolean isInfoWindowShown() {
		if (findMarker()) {
			return mKMarker.isInfoWindowShown();
		}
		return false;
	}

	/**
	 * 显示当前Marker信息窗口。
	 */
	public void showInfoWindow() {
		if (findMarker()) {
			mKMarker.showInfoWindow();
		}
	}

	/**
	 * 隐藏当前Marker信息窗口。
	 */
	public void hideInfoWindow() {
		if (findMarker()) {
			mKMarker.hideInfoWindow();
		}
	}

	
	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findMarker()) {
			return mKMarker.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findMarker()) {
			mKMarker.setLevel(level);
		}
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：2.0
	 */
	public float getZIndex() {
		if (findMarker()) {
			return mKMarker.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findMarker()) {
			mKMarker.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findMarker()) {
			return mKMarker.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findMarker()) {
			mKMarker.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findMarker()) {
			mKMarker.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mKMarker = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof Marker)) {
			return false;
		}
		
		Marker marker = (Marker) object;
		return marker.getId().equals(getId());
	}
	
}
