package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

/**
 * 标记覆盖物选项类。
 */
public final class MarkerOptions{
	LatLng m_position = new LatLng(0, 0);
	ArrayList<BitmapDescriptor> m_icons = new ArrayList<BitmapDescriptor>();
	int m_period = 50;
	float m_anchorU = 0.5f;
	float m_anchorV = 1.0f;
	float m_rotateAngle = 0;
	boolean m_draggable = false;
	boolean m_flat = false;
	boolean m_animatesDrop = false;
	String m_title = null;
	String m_snippet = null;
	int m_infoWindowOffsetX = 0;
	int m_infoWindowOffsetY = 0;
	int m_level = 0;
	float m_zIndex = 2.0f;
	boolean m_visible = true;
	boolean m_clickable = false;
	
	/**
	 * 返回显示位置坐标。
	 * @return 显示位置坐标
	 */
	public LatLng getPosition() {
		return m_position;
	}

	/**
	 * 设置显示位置坐标。
	 * @param position 显示位置坐标
	 * @return 该对象本身
	 */
	public MarkerOptions position(LatLng position) {
		m_position = new LatLng(position.latitude, position.longitude);
		return this;
	}

	/**
	 * 返回图片列表。
	 * <br>如果用户仅设置了一个图片，则返回一个图片。
	 * @return 图片列表
	 */
	public List<BitmapDescriptor> getIcons() {
		return m_icons;
	}

	/**
	 * 设置单个图片。
	 * @param icon 图片
	 * @return 该对象本身
	 */
	public MarkerOptions icon(BitmapDescriptor icon) {
		m_icons.clear();
		m_icons.add(icon);
		return this;
	}

	/**
	 * 设置多个图片，描点和大小以第一个图片为准，建议图片大小保持一致。
	 * @param icons 图片列表
	 * @return 该对象本身
	 */
	public MarkerOptions icons(List<BitmapDescriptor> icons) {
		m_icons.clear();
		m_icons.addAll(icons);
		return this;
	}

	/**
	 * 返回图片交替刷新周期。
	 * @return 图片交替刷新周期，单位：ms，默认：50ms
	 */
	public int getPeriod() {
		return m_period;
	}

	/**
	 * 设置图片交替刷新周期。
	 * <br>仅设置了多个图片才起作用。
	 * @param period 图片交替刷新周期，单位：ms，默认：50ms
	 * @return 该对象本身
	 */
	public MarkerOptions period(int period) {
		m_period = period;
		return this;
	}

	/**
	 * 返回锚点在水平范围的比例。
	 * @return 锚点在水平范围的比例，默认：0.5
	 */
	public float getAnchorU() {
		return m_anchorU;
	}

	/**
	 * 返回锚点垂直范围的比例。
	 * @return 锚点在垂直范围的比例，默认：1
	 */
	public float getAnchorV() {
		return m_anchorV;
	}

	/**
	 * 定义图片显示的锚点。锚点是图片接触地图平面的点。图片的左顶点为（0,0）点，右底点为（1,1）点。 默认为（0.5,1.0）
	 * @param u 锚点水平范围的比例，范围：[0,1]，默认：0.5
	 * @param v 锚点垂直范围的比例，范围：[0,1]，默认：1
	 * @return 该对象本身
	 */
	public MarkerOptions anchor(float u, float v) {
		m_anchorU = u;
		m_anchorV = v;
		return this;
	}
	
	/**
	 * 返回图片旋转角度
	 * @return 图片旋转角度，默认：0
	 */
	public float getRotateAngle() {
		return m_rotateAngle;
	}
	
	/**
	 * 设置图片旋转角度。
	 * @param angle 图片旋转角度，从正北开始，逆时针计算
	 * @return 该对象本身
	 */
	public MarkerOptions rotateAngle(float angle) {
		m_rotateAngle = angle;
		return this;
	}

	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public MarkerOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}

	/**
	 * 返回是否平贴地图显示。
	 * @return 是否平贴地图，默认：否
	 */
	public boolean isFlat() {
		return m_flat;
	}

	/**
	 * 设置是否平贴地图显示。
	 * @param flat 是否平贴地图，默认：否
	 * @return 该对象本身
	 */
	public MarkerOptions setFlat(boolean flat) {
		m_flat = flat;
		return this;
	}
	
	/**
	 * 返回是否支持下落动画效果。
	 * @return 是否支持下落动画效果，默认：否
	 */
	public boolean isAnimatesDrop() {
		return m_animatesDrop;
	}

	/**
	 * 设置是否支持下落动画效果。该属性仅在创建该覆盖物的时候以下落动画效果显示。
	 * @param animatesDrop 是否支持下落动画效果，默认：否
	 * @return 该对象本身
	 */
	public MarkerOptions setAnimatesDrop(boolean animatesDrop) {
		m_animatesDrop = animatesDrop;
		return this;
	}

	/**
	 * 返回标题信息。
	 * @return 标题信息
	 */
	public String getTitle() {
		return m_title;
	}

	/**
	 * 设置 标题信息。
	 * @param title 标题信息
	 * @return 该对象本身
	 */
	public MarkerOptions title(String title) {
		m_title = title;
		return this;
	}

	/**
	 * 返回描述信息。
	 * @return 描述信息
	 */
	public String getSnippet() {
		return m_snippet;
	}

	/**
	 * 设置 描述信息。
	 * @param snippet 描述信息
	 * @return 该对象本身
	 */
	public MarkerOptions snippet(String snippet) {
		m_snippet = snippet;
		return this;
	}

	/**
	 * 获取marker的水平偏移距离。
	 * @return 信息窗口相对原点（marker的中上点）的横向像素偏移量，默认：0
	 */
	public int getInfoWindowOffsetX() {
		return m_infoWindowOffsetX;
	}

	/**
	 * 获取marker的垂直偏移距离。
	 * @return 信息窗口相对原点（marker的中上点）的纵向像素偏移量，默认：0
	 */
	public int getInfoWindowOffsetY() {
		return m_infoWindowOffsetY;
	}

	/**
	 * 设置该Marker的信息窗口相对marker的偏移。
	 * <br>坐标系原点为marker的中上点，信息窗口相对此原点的像素偏移，向左和向上为负，向右和向下为正。
	 * 信息窗口的初始位置为marker上边线与信息窗口下边线重合，并且两者的中线在一条线上，此时offsetX和offsetX的值为0，及默认值。
	 * @param offsetX 信息窗口相对原点（marker的中上点）的横向像素偏移量，默认：0
	 * @param offsetY 信息窗口相对原点（marker的中上点）的纵向像素偏移量，默认：0
	 * @return 该对象本身
	 */
	public MarkerOptions setInfoWindowOffset(int offsetX, int offsetY) {
		m_infoWindowOffsetX = offsetX;
		m_infoWindowOffsetY = offsetY;
		return this;
	}

	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public MarkerOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：2.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：2.0
	 * @return 该对象本身
	 */
	public MarkerOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public MarkerOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}

	/**
	 * 返回是否可点击。
	 * @return 是否可点击，默认：否
	 */
	public boolean isClickable() {
		return m_clickable;
	}

	/**
	 * 设置是否可点击。
	 * @param enable 是否可点击，默认：否
	 * @return 该对象本身
	 */
	public MarkerOptions clickable(boolean clickable) {
		m_clickable = clickable;
		return this;
	}
}
