package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KNavigateArrow;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 箭头覆盖物类。
 * <p>拥有以下属性：</p>
 * <ul>
 * <li>顶点<br>箭头是由多个连贯的点构成的。</li>
 * <li>宽度<br>单位是像素，默认为10。</li>
 * <li>颜色<br>颜色是ARGB格式，参考{@link android.graphics.Color}，默认是黑色。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class NavigateArrow {
	Integer mId = KOverlayManager.assignId();
	KNavigateArrow mKArrow = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	NavigateArrow() {
	}
	
	private boolean findArrow() {
		if (mKArrow == null) {
			mKArrow = (KNavigateArrow) KOverlayManager.getInstance().findOverlay(getId());
		}
		
		return (mKArrow != null);
	}

	/**
	 * 返回顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改返回的顶点坐标列表不会影响对象内部的坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		ArrayList<LatLng> arr = new ArrayList<LatLng>();
		if (findArrow()) {
			List<KLatLng> kArr = mKArrow.getPoints();
			for (KLatLng kLatLng : kArr) {
				arr.add(kLatLng.toLatLng());
			}
		}
		return arr;
	}

	/**
	 * 设置顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改这个顶点坐标列表不会影响对象内部的坐标列表。
	 * @param points 顶点坐标列表
	 */
	public void setPoints(List<LatLng> points) {
		if (findArrow()) {
			ArrayList<KLatLng> arr = new ArrayList<KLatLng>();
			for (LatLng point : points) {
				arr.add(new KLatLng(point.latitude, point.longitude));
			}
			mKArrow.setPoints(arr);
		}
	}

	/**
	 * 返回箭头宽度
	 * @return 箭头宽度，单位：像素，默认：10像素
	 */
	public float getWidth() {
		if (findArrow()) {
			return mKArrow.getWidth();
		}
		return 0;
	}

	/**
	 * 设置箭头宽度
	 * @param width 箭头宽度，单位：像素
	 */
	public void setWidth(float width) {
		mKArrow.setWidth(width);
	}

	/**
	 * 返回箭颜色
	 * @return 箭头颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor() {
		if (findArrow()) {
			return mKArrow.getTopColor();
		}
		return 0;
	}

	/**
	 * 设置箭头颜色
	 * @param color 箭头颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setColor(int color) {
		if (findArrow()) {
			mKArrow.setTopColor(color);
			mKArrow.setSideColor(color);
		}
	}
	
	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findArrow()) {
			return mKArrow.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findArrow()) {
			mKArrow.setLevel(level);
		}
	}

	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findArrow()) {
			return mKArrow.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findArrow()) {
			mKArrow.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findArrow()) {
			return mKArrow.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findArrow()) {
			mKArrow.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findArrow()) {
			mKArrow.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mKArrow = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof NavigateArrow)) {
			return false;
		}
		
		NavigateArrow arrow = (NavigateArrow) object;
		return arrow.getId().equals(getId());
	}
	
}
