package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KOverlayManager;
import com.cmmap.internal.maps.model.KPolygon;

/**
 * 多边形覆盖物类。
 * <p>一个多边形可以是凸面体，也可以是凹面体,它有以下属性：</p>
 * <ul>
 * <li>边框<br>一个多边开的边框由一系列的顺时针或逆时针顶点组成。边框不需要声明起点和终点一致。如果边框的起点与终点不一致，API会自动将它封闭。</li>
 * <li>边框宽度<br>边框的宽度以像素为单位，不会受可视区域缩放级别的影响。默认为10。</li>
 * <li>边框颜色<br>边框颜色是一个ARGB格式，ARGB请参考android.graphics.Color。默认为黑色。</li>
 * <li>填充颜色<br>填充颜色是一个ARGB格式，ARGB格式可参考android.graphics.Color。默认是透明的。如果多边形的几何图形不正确，则填充颜色不会被绘制。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public final class Polygon {
	Integer mId = KOverlayManager.assignId();
	KPolygon mkPolygon = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Polygon() {
	}
	
	private boolean findPolygon() {
		if (mkPolygon == null) {
			mkPolygon = (KPolygon) KOverlayManager.getInstance().findOverlay(getId());
		}
		
		return (mkPolygon != null);
	}

	/**
	 * 返回顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改返回的顶点坐标列表不会影响对象内部的坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		ArrayList<LatLng> points = new ArrayList<LatLng>();
		if (findPolygon()) {
			for (KLatLng point : mkPolygon.getPoints()) {
				points.add(point.toLatLng());
			}
		}
		return points;
	}

	/**
	 * 设置顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改这个顶点坐标列表不会影响对象内部的坐标列表。
	 * @param points 顶点坐标列表
	 */
	public void setPoints(List<LatLng> points) {
		if (findPolygon()) {
			ArrayList<KLatLng> kPoints = new ArrayList<KLatLng>();
			for (LatLng point : points) {
				kPoints.add(new KLatLng(point.latitude, point.longitude));
			}
			mkPolygon.setPoints(kPoints);
		}
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth() {
		if (findPolygon()) {
			return mkPolygon.getStrokeWidth();
		}
		return 0;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素
	 */
	public void setStrokeWidth(float width) {
		if (findPolygon()) {
			mkPolygon.setStrokeWidth(width);
		}
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		if (findPolygon()) {
			return mkPolygon.getStrokeColor();
		}
		return 0;
	}

	/**
	 * 设置边框颜色。
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setStrokeColor(int color) {
		if (findPolygon()) {
			mkPolygon.setStrokeColor(color);
		}
	}

	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor() {
		if (findPolygon()) {
			return mkPolygon.getFillColor();
		}
		return 0;
	}

	/**
	 * 设置填充那个颜色。
	 * @param color 填充颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setFillColor(int color) {
		if (findPolygon()) {
			mkPolygon.setFillColor(color);
		}
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findPolygon()) {
			return mkPolygon.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findPolygon()) {
			mkPolygon.setDraggable(enable);
		}
	}
	
	/**
	 * 判断多边形是否包含传入的经纬度点。
	 * @param latLng 传入的经纬度点
	 * @return 是否包含
	 */
	public boolean contains(LatLng latLng) {
		if (findPolygon()) {
			return mkPolygon.contains(new KLatLng(latLng.latitude, latLng.longitude));
		}
		return false;
	}

	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findPolygon()) {
			return mkPolygon.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findPolygon()) {
			mkPolygon.setLevel(level);
		}
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findPolygon()) {
			return mkPolygon.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findPolygon()) {
			mkPolygon.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findPolygon()) {
			return mkPolygon.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findPolygon()) {
			mkPolygon.setVisible(visible);
		}
	}

	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findPolygon()) {
			mkPolygon.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mkPolygon = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null || 
				!(object instanceof Polygon)) {
			return false;
		}
		
		Polygon polygon = (Polygon) object;
		return polygon.getId().equals(getId());
	}
	
}
