package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Color;

/**
 * 多边形覆盖物选项类
 */
public final class PolygonOptions {
	ArrayList<LatLng> m_points = new ArrayList<LatLng>();
	float m_strokeWidth = 10;
	int m_strokeColor = Color.BLACK;
	int m_fillColor = Color.BLACK;
	boolean m_draggable = false;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	
	/**
	 * 返回顶点坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		return m_points;
	}

	/**
	 * 添加一个顶点到顶点坐标列表中。
	 * @param point 要添加的顶点
	 * @return 该对象本身
	 */
	public PolygonOptions add(LatLng point) {
		m_points.add(point);
		return this;
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public PolygonOptions add(LatLng... points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public PolygonOptions addAll(Iterable<LatLng> points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth() {
		return m_strokeWidth;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，默认：10像素
	 * @return 该对象本身
	 */
	public PolygonOptions strokeWidth(float width) {
		m_strokeWidth = width;
		return this;
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		return m_strokeColor;
	}

	/**
	 * 设置边框颜色。
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public PolygonOptions strokeColor(int color) {
		m_strokeColor = color;
		return this;
	}

	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor() {
		return m_fillColor;
	}

	/**
	 * 设置填充颜色。
	 * @param color 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public PolygonOptions fillColor(int color) {
		m_fillColor = color;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public PolygonOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public PolygonOptions level(int level) {
		m_level = level;
		return this;
	}

	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public PolygonOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public PolygonOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
