package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KOverlayManager;
import com.cmmap.internal.maps.model.KPolyline;

/**
 * 线段覆盖物类。
 * <p>一个线段是多个连贯点的集合线段，它拥有以下属性：</p>
 * <ul>
 * <li>顶点<br>线段是由两个顶点之间连贯的点构成的。如果两个顶点相同，则一个线段将闭合。</li>
 * <li>宽度<br>宽度是单位是像素。宽度是可视区域的缩放级别无关。默认为10。</li>
 * <li>颜色<br>线段的颜色是ARGB格式，颜色格式可以参考android.graphics.Color。默认是黑色(0xff000000)。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class Polyline {
	Integer mId = KOverlayManager.assignId();
	KPolyline mkPolyline = null;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Polyline() {
	}
	
	private boolean findPolyline() {
		if (mkPolyline == null) {
			mkPolyline = (KPolyline) KOverlayManager.getInstance().findOverlay(getId());
		}
		
		return (mkPolyline != null);
	}

	/**
	 * 返回顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改返回的顶点坐标列表不会影响对象内部的坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		ArrayList<LatLng> points = new ArrayList<LatLng>();
		if (findPolyline()) {
			for (KLatLng point : mkPolyline.getPoints()) {
				points.add(point.toLatLng());
			}
		}
		return points;
	}

	/**
	 * 设置线段的顶点。如果调用此方法后修改了List的值将不会影响到线段。
	 * @param points 线段顶点的经纬度集合
	 */
	public void setPoints(List<LatLng> points) {
		if (findPolyline()) {
			ArrayList<KLatLng> kPoints = new ArrayList<KLatLng>();
			for (LatLng point : points) {
				kPoints.add(new KLatLng(point.latitude, point.longitude));
			}
			mkPolyline.setPoints(kPoints);
		}
	}

	/**
	 * 返回线宽。
	 * @return 线宽，单位：像素，默认：10像素
	 */
	public float getWidth() {
		if (findPolyline()) {
			return mkPolyline.getWidth();
		}
		return 0;
	}

	/**
	 * 设置线宽。
	 * @param width 线宽，单位：像素
	 */
	public void setWidth(float width) {
		if (findPolyline()) {
			mkPolyline.setWidth(width);
		}
	}

	/**
	 * 返回线的颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor() {
		if (findPolyline()) {
			return mkPolyline.getColor();
		}
		return 0;
	}

	/**
	 * 设置线的颜色
	 * @param color 线的颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setColor(int color) {
		if (findPolyline()) {
			mkPolyline.setColor(color);
		}
	}

	/**
	 * 返回是否虚线绘制，默认：否。
	 * @return 是否虚线绘制
	 */
	public boolean isDottedLine() {
		if (findPolyline()) {
			return mkPolyline.isDottedLine();
		}
		return false;
	}

	/**
	 * 设置是否虚线绘制。
	 * @param isDottedLine 是否虚线绘制
	 */
	public void setDottedLine(boolean isDottedLine) {
		if (findPolyline()) {
			mkPolyline.setDottedLine(isDottedLine);
		}
	}

	/**
	 * 返回是否以大地曲线绘制，默认：否。
	 * @return 是否以大地曲线绘制
	 */
	public boolean isGeodesic() {
		if (findPolyline()) {
			return mkPolyline.isGeodesic();
		}
		return false;
	}

	/**
	 * 设置是否以大地曲线绘制。
	 * @param isGeodesic 是否以大地曲线绘制
	 */
	public void setGeodesic(boolean isGeodesic) {
		if (findPolyline()) {
			mkPolyline.setGeodesic(isGeodesic);
		}
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		if (findPolyline()) {
			return mkPolyline.isDraggable();
		}
		return false;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean enable) {
		if (findPolyline()) {
			mkPolyline.setDraggable(enable);
		}
	}

	
	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findPolyline()) {
			return mkPolyline.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findPolyline()) {
			mkPolyline.setLevel(level);
		}
	}

	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findPolyline()) {
			return mkPolyline.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findPolyline()) {
			mkPolyline.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findPolyline()) {
			return mkPolyline.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findPolyline()) {
			mkPolyline.setVisible(visible);
		}
	}
	
	/**
	 * 设置名称是否可见。
	 * @param visible 是否可见
	 */
	public void setLineNameVisible(boolean visible) {
		if (findPolyline()) {
			mkPolyline.setLineNameVisible(visible);
		}
	}
	
	/**
	 * 返回名称是否可见。
	 * @return visible 是否可见
	 */
	public void getLineNameVisible() {
		if (findPolyline()) {
			mkPolyline.getLineNameVisible();
		}
	}

	/**
	 * 返回名称前景色。
	 * @return 颜色，默认：黑色
	 */
	public int getNameColor() {
		if (findPolyline()) {
			return mkPolyline.getNameColor();
		}
		return 0;
	}
	
	/**
	 * 设置名称前景色。
	 * @param 颜色，默认：黑色
	 * @return 无
	 */
	public void nameColor(int color) {
		if (findPolyline()) {
			mkPolyline.nameColor(color);
		}
	}
	
	/**
	 * 返回名称边缘色。
	 * @return 颜色，默认：不色
	 */
	public int getNameEdgeColor() {
		if (findPolyline()) {
			return mkPolyline.getNameEdgeColor();
		}
		return 0;
	}
	
	/**
	 * 设置名称背景色。
	 * @param 颜色，默认：黑色
	 * @return 无
	 */
	public void nameEdgeColor(int color) {
		if (findPolyline()) {
			mkPolyline.nameEdgeColor(color);
		}
	}
	
	/**
	 * 返回名称大小。
	 * @return 字体大小，默认：22
	 */
	public int getNameFontSize() {
		if (findPolyline()) {
			return mkPolyline.getNameFontSize();
		}
		return 0;
	}
	
	/**
	 * 设置名称大小。
	 * @param 字体大小，默认：22
	 * @return 无
	 */
	public void nameFontSize(int size) {
		if (findPolyline()) {
			mkPolyline.nameFontSize(size);
		}
	}
	
	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该覆盖物。
	 */
	public void remove() {
		if (findPolyline()) {
			mkPolyline.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mkPolyline = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof Polyline)) {
			return false;
		}
		
		Polyline polyline = (Polyline) object;
		return polyline.getId().equals(getId());
	}
	
}
