package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.internal.maps.model.KPolylineOptions;

import android.graphics.Color;

/**
 * 线段覆盖物选项类
 */
public final class PolylineOptions {
	ArrayList<LatLng> m_points = new ArrayList<LatLng>();
	float m_width = 10;
	int m_color = Color.BLACK;
	boolean m_useTexture = false;
	BitmapDescriptor m_customTexture = null;
	boolean m_dottedLine = false;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	boolean m_is_Geodesic = false;
	boolean m_draggable = false;
	boolean m_line_name_visible = false;
	String m_line_name = "";
	int m_name_color = Color.BLACK;;
	int m_name_edge_color = Color.WHITE;
	int m_name_font_size = 22;
    private List<Integer> m_lst_colors = new ArrayList<Integer>();
    private List<BitmapDescriptor> m_lst_bitmapDescriptors = new ArrayList<BitmapDescriptor>();
    private List<Integer> m_lst_texture_index = new ArrayList<Integer>();
    private List<String> m_lst_line_name = new ArrayList<String>();
    private List<Integer> m_lst_name_index = new ArrayList<Integer>();
	
	/**
	 * 返回顶点坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		return m_points;
	}

	/**
	 * 添加一个顶点到顶点坐标列表中。
	 * @param point 要添加的顶点
	 * @return 该对象本身
	 */
	public PolylineOptions add(LatLng point) {
		m_points.add(point);
		return this;
	}

	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public PolylineOptions add(LatLng... points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public PolylineOptions addAll(Iterable<LatLng> points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 返回线宽。
	 * @return 线宽，单位：像素，默认：10像素
	 */
	public float getWidth() {
		return m_width;
	}

	/**
	 * 设置线宽。
	 * @param width 线宽，单位：像素，默认：10像素
	 * @return 该对象本身
	 */
	public PolylineOptions width(float width) {
		m_width = width;
		return this;
	}
	
	/**
	 * 返回名称前景色。
	 * @return 颜色，默认：黑色
	 */
	public int getNameColor() {
		return m_name_color;
	}
	
	/**
	 * 设置名称前景色。
	 * @param 颜色，默认：黑色
	 * @return 该对象本身
	 */
	public PolylineOptions nameColor(int color) {
		m_name_color = color;
		return this;
	}
	
	/**
	 * 返回名称边缘色。
	 * @return 颜色，默认：不色
	 */
	public int getNameEdgeColor() {
		return m_name_edge_color;
	}
	
	/**
	 * 设置名称背景色。
	 * @param 颜色，默认：黑色
	 * @return 该对象本身
	 */
	public PolylineOptions nameEdgeColor(int color) {
		m_name_edge_color = color;
		return this;
	}
	
	/**
	 * 返回名称大小。
	 * @return 字体大小，默认：22
	 */
	public int getNameFontSize() {
		return m_name_font_size;
	}
	
	/**
	 * 设置名称大小。
	 * @param 字体大小，默认：22
	 * @return 该对象本身
	 */
	public PolylineOptions nameFontSize(int size) {
		m_name_font_size = size;
		return this;
	}
		
	/**
	 * 返回线的颜色。
	 * @return 线的颜色， ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor() {
		return m_color;
	}

	/**
	 * 设置线的颜色.
	 * @param color 线的颜色， ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public PolylineOptions color(int color) {
		m_color = color;
		return this;
	}

	/**
	 * 返回是否使用纹理绘制。
	 * @return 是否使用纹理绘制，默认：不使用纹理绘制
	 */ 
	public boolean isUseTexture() {
		return m_useTexture;
	}

	/**
	 * 设置是否使用纹理绘制。
	 * @param useTexture 是否使用纹理绘制，默认：不使用纹理绘制
	 * @return 该对象本身
	 */
	public PolylineOptions setUseTexture(boolean useTexture) {
		m_useTexture = useTexture;
		return this;
	}

	/**
	 * 返回纹理图片。
	 * @return 纹理图片，默认：空
	 */
	public BitmapDescriptor getCustomTexture() {
		return m_customTexture;
	}

	/**
	 * 设置纹理图片。
	 * @param customTexture 纹理图片，默认：空
	 * @return 该对象本身
	 */
	public PolylineOptions setCustomTexture(BitmapDescriptor customTexture) {
		m_customTexture = customTexture;
		return this;
	}
	
	/**
	 * 是否虚线绘制，默认：不使用虚线绘制。
	 * @return 是否虚线绘制
	 */
	public boolean isDottedLine() {
		return m_dottedLine;
	}

	/**
	 * 是否虚线绘制。
	 * @param isDottedLine 是否虚线绘制，默认：否
	 * @return 该对象本身
	 */
	public PolylineOptions setDottedLine(boolean isDottedLine) {
		m_dottedLine = isDottedLine;
		return this;
	}
	
	/**
	 * 返回是否以大地曲线绘制，默认：否。
	 * @return 是否以大地曲线绘制
	 */
	public boolean isGeodesic() {
		return m_is_Geodesic;
	}
	
	/**
	 * 设置名称。
	 * @param lineName 路线名称
	 * @return 该对象本身
	 */
	public PolylineOptions setLineName(String lineName) {
		m_line_name = lineName;
		return this;
	}

	/**
	 * 设置名称是否可见。
	 * @param visible 是否可见
	 * @return 该对象本身
	 */
	public PolylineOptions setLineNameVisible(boolean visible) {
		m_line_name_visible = visible;
		return this;
	}	
	
	/**
	 * 返回名称。
	 * @return String 名称
	 */
	public String getLineName() {
		return m_line_name;
	}
	
	/**
	 * 返回名称是否可见。
	 * @return visible 是否可见
	 */
	public boolean getLineNameVisible() {
		return m_line_name_visible;
	}
	
	/**
	 * 设置是否以大地曲线绘制。
	 * @param isGeodesic 是否以大地曲线绘制，默认：否
	 * @return 该对象本身
	 */
	public PolylineOptions setGeodesic(boolean isGeodesic) {
		m_is_Geodesic = isGeodesic;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public PolylineOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}

	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public PolylineOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public PolylineOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public PolylineOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
	/**
	 * 获取线段的颜色列表
	 * @return 颜色列表
	 */
	public List<Integer> getColorValues() {
	    return m_lst_colors;
	}
	
	/**
	 * 获取线段纹理列表
	 * @return 纹理列表
	 */
	public List<BitmapDescriptor> getCustomTextureList(){
        return m_lst_bitmapDescriptors;
    }
	
	/**
	 * 获取线段纹理index列表
	 * @return 分段纹理index列表
	 */
	public List<java.lang.Integer> getCustomTextureIndex(){
        return m_lst_texture_index;
    }
	
	/**
	 * 获取线段纹理列表
	 * @return 纹理列表
	 */
	public List<String> getLineNameList(){
        return m_lst_line_name;
    }
	
	/**
	 * 获取线段纹理index列表
	 * @return 分段纹理index列表
	 */
	public List<java.lang.Integer> getLineNameIndex(){
        return m_lst_name_index;
    }
	
	
	/**
	 * 设置线段的颜色
	 * @param colors 颜色列表
	 * @return PolylineOptions 对象
	 */
	public PolylineOptions colorValues(List<Integer> colors) {
		m_lst_colors = colors;
	    return this;
	}
	
	/**
	 * 设置线段纹理list
	 * @param bitmapDescriptors 纹理列表
	 * @return PolylineOptions 对象
	 */
	public PolylineOptions setCustomTextureList(List<BitmapDescriptor> bitmapDescriptors) {
        this.m_lst_bitmapDescriptors = bitmapDescriptors;
        return this;
    }
	
	/**
	 * 设置线段纹理index数组
	 * @param integers 每一段对应的纹理，用纹理列表的index来对应
	 * @return PolylineOptions 对象
	 */
	public PolylineOptions setCustomTextureIndex(List<Integer> integers) {
        this.m_lst_texture_index = integers;
        return this;
    }
	
	/**
	 * 设置线段名称list
	 * @param bitmapDescriptors 名称列表
	 * @return PolylineOptions 对象
	 */
	public PolylineOptions setLineNameList(List<String> lineNames) {
        this.m_lst_line_name = lineNames;
        return this;
    }
	
	/**
	 * 设置线段名称index数组
	 * @param integers 每一段对应的名称，用名称列表的index来对应
	 * @return PolylineOptions 对象
	 */
	public PolylineOptions setLineNameIndex(List<Integer> integers) {
        this.m_lst_name_index = integers;
        return this;
    }
}
