package com.cmmap.api.maps.model;

import com.cmmap.internal.maps.model.KOverlayManager;
import com.cmmap.internal.maps.model.KTileOverlay;

/**
 * 栅格图层类。
 * <p>栅格图层是显示在基本地图上的一组图像。栅格图层具有以下属性：</p>
 * <ul> 
 * <li>栅格提供者<br>提供者提供在栅格图层上使用的图像。
 * 您必须在栅格添加到地图上之前指定提供者。
 * 一旦栅格添加到地图之后，提供者将不能更改。
 * 如果栅格提供者发生变化，您必须使用{@link #clearTileCache}方法确保以前的栅格图不再出现。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 */
public final class TileOverlay {
	KTileOverlay mkTileOverlay = null;
	Integer mId = KOverlayManager.assignId();
	/**
	 * 内部构造函数，不公开提供
	 */
	TileOverlay() {
	}
	
	private boolean findTileOverlay() {
		if (mkTileOverlay == null) {
			mkTileOverlay = (KTileOverlay) KOverlayManager.getInstance().findOverlay(getId());
		}
		
		return (mkTileOverlay != null);
	}

	/**
	 * 清空栅格图层的缓存。
	 */
	public void clearTileCache() {
		if (findTileOverlay()) {
			mkTileOverlay.clearTileCache();
		}
	}

	
	/**
	 * 返回显示层次。
	 * @return 显示层次，默认：0
	 */
	public int getLevel() {
		if (findTileOverlay()) {
			return mkTileOverlay.getLevel();
		}
		return 0;
	}

	/**
	 * 设置显示层次
	 * @param level
	 */
	public void setLevel(int level) {
		if (findTileOverlay()) {
			mkTileOverlay.setLevel(level);
		}
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		if (findTileOverlay()) {
			mkTileOverlay.getZIndex();
		}
		return 0;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引
	 */
	public void setZIndex(float zIndex) {
		if (findTileOverlay()) {
			mkTileOverlay.setZIndex(zIndex);
		}
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		if (findTileOverlay()) {
			return mkTileOverlay.isVisible();
		}
		return false;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见
	 */
	public void setVisible(boolean visible) {
		if (findTileOverlay()) {
			mkTileOverlay.setVisible(visible);
		}
	}

	/**
	 * 返回图层ID。
	 * <br>ID是该图层在所属的地图对象里的唯一标识。
	 * @return ID 图层ID
	 */
	public String getId() {
		return mId.toString();
	}

	/**
	 * 从地图对象中删除该图层。
	 */
	public void remove() {
		if (findTileOverlay()) {
			mkTileOverlay.remove();
		}
		KOverlayManager.getInstance().removeOverlay(getId());
		KOverlayManager.getInstance().removeTileProvider(getId());
		KOverlayManager.getInstance().removeOverlay(this);
		mkTileOverlay = null;
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + mId; 
		return result;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null || !(object instanceof TileOverlay)) {
			return false;
		}
		
		TileOverlay obj = (TileOverlay) object;
		return obj.getId().equals(getId());
	}
}
