package com.cmmap.api.maps.model;

import java.io.File;

import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * 栅格图层选项类。
 * <p>使用者使用该类添加栅格图层，参见{@link com.cmmap.api.maps.Map#addTileOverlay}。</p>
 */
public final class TileOverlayOptions {
	TileProvider m_tileProvider = null;
	String m_DiskCacheDir = KOverlayManager.getDefaultTileDiskCachePath();
	boolean m_DiskCacheEnabled = true;
	int m_DiskCacheSize = 20480;
	int m_MemCacheSize = 5120;
	boolean m_clearOldCache = true; 
	boolean m_MemoryCacheEnabled = true;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	
	/**
	 * 返回栅格图层提供者。
	 * @return 栅格图层提供者
	 */
	public TileProvider getTileProvider() {
		return m_tileProvider;
	}

	/**
	 * 设置栅格图层提供者。
	 * <br>这里的栅格图层提供者指的是{@link TileProvider}或者{@link UrlTileProvider}
	 * @param tileProvider 栅格图层提供者
	 * @return 该对象本身
	 */
	public TileOverlayOptions tileProvider(TileProvider tileProvider) {
		m_tileProvider = tileProvider;
		return this;
	}

	/**
	 * 返回磁盘缓存目录。
	 * @return 磁盘缓存目录
	 */
	public String getDiskCacheDir() {
		return m_DiskCacheDir;
	}

	/**
	 * 设置磁盘缓存目录。
	 * @param path 磁盘缓存目录，绝对路径
	 * @return 该对象本身
	 */
	public TileOverlayOptions diskCacheDir(String path) {
		if (path != null) {
			File file = new File(path);
			if(file.exists() || file.mkdirs()){
				m_DiskCacheDir = path;
			}
		}
		return this;
	}

	/**
	 * 返回是否开启磁盘缓存。
	 * @return 是否开启磁盘缓存，默认：开启
	 */
	public boolean getDiskCacheEnabled() {
		return m_DiskCacheEnabled;
	}

	/**
	 * 设置是否开启磁盘缓存。
	 * @param enabled 是否开启磁盘缓存，默认：开启
	 * @return 该对象本身
	 */
	public TileOverlayOptions diskCacheEnabled(boolean enabled) {
		m_DiskCacheEnabled = enabled;
		return this;
	}

	/**
	 * 返回磁盘缓存大小。
	 * @return 磁盘缓存大小，单位：KB，默认：20*1024KB
	 */
	public int getDiskCacheSize() {
		return m_DiskCacheSize;
	}

	/**
	 * 设置磁盘缓存大小。
	 * @param kiloBytes 磁盘缓存大小，单位：KB，默认：20*1024KB
	 * @return 该对象本身
	 */
	public TileOverlayOptions diskCacheSize(int kiloBytes) {
		m_DiskCacheSize = kiloBytes;
		return this;
	}

	/**
	 * 返回内存缓存大小。
	 * @return 内存缓存大小，单位KB，默认值5*1024KB
	 */
	public int getMemCacheSize() {
		return m_MemCacheSize;
	}

	/**
	 * 设置内存缓存大小。
	 * @param kiloBytes 内存缓存大小，单位KB，默认值5*1024KB
	 * @return 该对象本身
	 */
	public TileOverlayOptions memCacheSize(int kiloBytes) {
		m_MemCacheSize = kiloBytes;
		return this;
	}

	/**
	 * 获取是否开启内存缓存。
	 * @return 是否开启内存缓存，默认：开启
	 */
	public boolean getMemCacheEnabled() {
		return m_MemoryCacheEnabled;
	}

	/**
	 * 设置是否开启内存缓存。
	 * @param enabled 是否开启内存缓存，默认：开启
	 * @return 该对象本身
	 */
	public TileOverlayOptions memCacheEnabled(boolean enabled) {
		m_MemoryCacheEnabled = enabled;
		return this;
	}
	
	/**
	 * 返回是否清除旧缓存。
	 * @return 是否清除旧缓存，默认：开启
	 */
	public boolean isClearOldCache() {
		return m_clearOldCache;
	}
	
	/**
	 * 是否清除旧缓存。
	 * <p>如果用户设置清除或者默认，且对应的缓存目录已经存在，则删除之前的旧数据。
	 * @param enabled 否清除旧缓存，默认：开启
	 * @return 该对象本身
	 */
	public TileOverlayOptions setClearOldCache(boolean enabled) {
		m_clearOldCache = enabled;
		return this;
	}
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public TileOverlayOptions level(int level) {
		m_level = level;
		return this;
	}

	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public TileOverlayOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}
	
	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public boolean isVisible() {
		return m_visible;
	}

	
	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public TileOverlayOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
