package com.cmmap.internal.maps;

import com.cmmap.api.maps.CameraUpdate;
import com.cmmap.api.maps.Map;
import com.cmmap.api.maps.model.CameraPosition;
import com.cmmap.api.maps.model.LatLngBounds;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图可视范围更新创建类。
 * <p>该类用于创建{@link CameraUpdate}对象，
 * 主要用于用户调用{@link Map#moveCamera}和{@link Map#animateCamera}修改地图可视范围。</p>
 */
public class KCameraUpdateFactory {

	/**
	 * 返回一个{@link CameraUpdate}对象，只改变旋转角度。
	 * @param bearing 地图旋转角度，正北方向到地图方向逆时针旋转的角度，单位：角度，范围：[0,360)
	 * @return 一个包含旋转角度改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate changeBearing(float bearing) {
		KCameraUpdate update = new KCameraUpdate();
		update.targetPosition.bearing = bearing;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_Bearing_Only;
		return update;
	}

	/**
	 * 返回一个CameraUpdate对象，只改变地图可视区域中心点，地图缩放级别不变。
	 * 
	 * @param latLng 地图中心点坐标
	 * @return 一个包含地图中心点坐标改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate changeLatLng(KLatLng latLng) {
		KCameraUpdate update = new KCameraUpdate();
		update.targetPosition.target = latLng;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_LatLng_Only;
		return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，只改变地图倾斜度。
	 * @param tilt 地图倾斜度，单位：角度，范围：[0,45]。
	 * @return 一个包含地图倾斜度改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate changeTilt(float tilt) {
		KCameraUpdate update = new KCameraUpdate();
		update.targetPosition.tilt = tilt;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_Tilt_Only;
		return update;
	}

	/**
	 * 返回一个定义了可视区域移动位置CameraUpdate 对象
	 * 
	 * @param cameraPosition
	 * @return
	 */
	public static KCameraUpdate newCameraPosition(KCameraPosition cameraPosition) {
		KCameraUpdate update = new KCameraUpdate(cameraPosition);
		return update;
	}
	/**
	 * 返回一个定义了地图可视范围的{@link CameraUpdate}对象。
	 * @param camera 地图可视范围对象。
	 * @return 一个包含地图可视范围的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate newCameraPosition(CameraPosition cameraPosition) {
		KCameraUpdate update = new KCameraUpdate(cameraPosition);
		return update;
	}
	
	public static KCameraUpdate newLatLngBounds(LatLngBounds bounds, int padding) {
		return new KCameraUpdate(bounds, -1, -1, padding);
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，包含一个经纬度限制区域，并且是最大可能的缩放级别，地图旋转角度和倾斜角度为0。
	 * @param bounds 经纬度限制区域
	 * @param leftPadding 经纬度限制区域和地图视图之间的左边空白距离，单位像素
	 * @param rightPadding 经纬度限制区域和地图视图之间的右边空白距离，单位像素
	 * @param topPadding 经纬度限制区域和地图视图之间的上边空白距离，单位像素
	 * @param bottomPadding 经纬度限制区域和地图视图之间的下边空白距离，单位像素
	 * @return 一个包含经纬度限制区域的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate newLatLngBounds(LatLngBounds bounds, int width, int height, int padding) {
		return new KCameraUpdate(bounds, width, height, padding);
	}
	
	/**
	 * 返回一个{@link KCameraUpdate}对象，包含一个经纬度限制区域，并且是最大可能的缩放级别，地图旋转角度和倾斜角度为0。
	 * @param bounds 经纬度限制区域
	 * @param leftPadding 经纬度限制区域和地图视图之间的左边空白距离，单位像素
	 * @param rightPadding 经纬度限制区域和地图视图之间的右边空白距离，单位像素
	 * @param topPadding 经纬度限制区域和地图视图之间的上边空白距离，单位像素
	 * @param bottomPadding 经纬度限制区域和地图视图之间的下边空白距离，单位像素
	 * @return 一个包含经纬度限制区域的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate newLatLngBounds(LatLngBounds bounds,
			int leftPadding, int rightPadding, int topPadding, int bottomPadding) {
		return new KCameraUpdate(bounds, leftPadding, rightPadding, topPadding, bottomPadding);
	}

	/**
	 * 返回一个包含缩放级别增大的CameraUpdate 对象，调用此方法一次缩放级别加大一级，也就是屏幕距离地面更近一级
	 * 
	 * @return 一个包含缩放级别改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate zoomIn() {
		KCameraUpdate update = new KCameraUpdate();
		float zoom = calcZoom(update.targetPosition.zoom, true);
		update.targetPosition.zoom = zoom;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_Zoom_Only;
		return update;
	}

	static private float calcZoom(float zoom, boolean is_zoom_in) {
		float result;
		int i_zoom;
		if (is_zoom_in) {
			// zoom += 1.0
			i_zoom = (int)(zoom + 1.0);
			int max = MapCore.GetMaxScale();
			if (i_zoom > max) {
				result = max;
			}else {
				result = i_zoom;
			}
		}else {
			// zoom -= 1.0
			if (zoom - (int)(zoom) > 0.1) {
				i_zoom = (int)(zoom);
			}else {
				i_zoom = (int)(zoom - 1.0);
			}
			
			int min = MapCore.GetMinScale();
			if (i_zoom < min) {
				result = min;
			}else {
				result = i_zoom;
			}
		}
		return result;
	}
	
	/**
	 * 返回一个包含缩放级别减小的CameraUpdate 对象，调用此方法一次缩放级别的减小一级，也就是屏幕距离地面更远一级
	 * 
	 * @return 一个包含缩放级别改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate zoomOut() {
		KCameraUpdate update = new KCameraUpdate();
		float zoom = calcZoom(update.targetPosition.zoom, false);
		update.targetPosition.zoom = zoom;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_Zoom_Only;
		return update;
	}

	/**
	 * 返回一个{@link CameraUpdate}对象，包括地图中心点坐标以及缩放级别，地图旋转角度和倾斜角度为0。
	 * @param latLng 地图中心点坐标
	 * @param zoom 地图缩放级别
	 * @return 一个包含地图中心点坐标和缩放级的@link CameraUpdate}对象
	 */
	public static KCameraUpdate newLatLngZoom(KLatLng latLng, float zoom) {
		KCameraUpdate update = new KCameraUpdate();
		update.targetPosition.target = latLng;
		update.targetPosition.zoom = zoom;
		update.m_byUpdateType &= (KCameraUpdate.Camera_Update_LatLng_Only | KCameraUpdate.Camera_Update_Zoom_Only);
		return update;

	}

	/**
	 * 返回一个包含缩放级别改变的{@link CameraUpdate}对象。
	 * @param zoom 地图缩放级别
	 * @return 一个包含地图缩放级别改变的{@link CameraUpdate}对象
	 */
	public static KCameraUpdate zoomTo(float zoom) {
		KCameraUpdate update = new KCameraUpdate();
		update.targetPosition.zoom = zoom;
		update.m_byUpdateType &= KCameraUpdate.Camera_Update_Zoom_Only;
		return update;
	}
}
