package com.cmmap.internal.maps;

import java.util.concurrent.CopyOnWriteArrayList;

import android.content.Context;
import android.graphics.Point;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;

import com.cmmap.internal.driver.view.IRefreshListener;
import com.cmmap.internal.driver.view.ViewContext;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.KMap.InfoView;
import com.cmmap.internal.maps.model.KArc;
import com.cmmap.internal.maps.model.KCircle;
import com.cmmap.internal.maps.model.KGroundOverlay;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KPolygon;
import com.cmmap.internal.maps.model.KPolyline;
import com.cmmap.internal.maps.model.KShapeOverlay;

/**
 * Marker所在View，不对外开放，外部不可见
 *
 */
public class KInfoOverlayView extends FrameLayout implements IRefreshListener {
	private static final String TAG = "InfoOverlayView";
	private KMap map;
	private ViewContext viewContext;
	
	private InfoView touchOverlay;
	private InfoView touchInfo;
	private KShapeOverlay touchShapeOverlay;
	private boolean bMoveOperate = false;
	
	private int iDispatchTouchEventCheck = -1;
	
	static final long MARKER_LONG_PRESS_TIME = 500;
	private long downTick = 0;
	private Point downPoint = null;
	private static final int MOVE_GESTURE_BE_INVALID_PIXEL_DIS = 10;
	private Point lastMovePoint = null;
	
	private KMarker m_infowindowSubtitude;
	private static final float FLOAT_ACCURACY = 0.000001f;
	
	CopyOnWriteArrayList<InfoView> m_infoViews = null;
	CopyOnWriteArrayList<KShapeOverlay> m_shapeOverlays = null;
	
	public KInfoOverlayView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	public KInfoOverlayView(Context context) {
		super(context);
	}

	public KInfoOverlayView(Context context, KMap map, ViewContext viewContext) {
		super(context);
		this.map = map;
		this.viewContext = viewContext;
	};
	
	
	@Override
	protected void onLayout(boolean changed, int left, int top, int right,
			int bottom) {
		super.onLayout(changed, left, top, right, bottom);
		
		CopyOnWriteArrayList<InfoView> infoViews = map.getInfoViews();
		synchronized (infoViews) {
			for (InfoView info : infoViews) {
				info.recalcPos();
			}
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		m_infoViews = map.getInfoViews();
		m_shapeOverlays = map.getShapeOverlays();
		if(iDispatchTouchEventCheck > 0)
		{
			return false;
		}
		// infowindow子控件没有处理touchEvent，父控件再次调用onTouchEvent，截获
		else if(iDispatchTouchEventCheck == 0)
		{
			if((event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_UP)
			{
				synchronized (m_infoViews) {
					map.onInfoWindowClick(touchInfo.marker);
					touchInfo = null;					
				}
			}
			return true;
		}
		iDispatchTouchEventCheck++;
		
		boolean bHitInfoOverlay = hitInfoOverlay(event);
		boolean bHitShapeOverlay = hitShapeOverlay(event);
		if (bHitInfoOverlay || bHitShapeOverlay) {
			synchronized (m_infoViews) {
				// marker和形状overlay都点击到，根据显示层级选取
				if (touchOverlay != null && touchOverlay.marker != null && touchShapeOverlay != null) {
					float dZIndex = touchOverlay.marker.getZIndex() - touchShapeOverlay.getZIndex();
					if (dZIndex > FLOAT_ACCURACY
							|| (Math.abs(dZIndex) <= FLOAT_ACCURACY && touchOverlay.marker.getOverlayId() > touchShapeOverlay.getOverlayId())){
						touchShapeOverlay = null;
					} else {
						touchOverlay = null;
					}
				}
			}
			downTick = System.currentTimeMillis();
			downPoint = new Point((int)event.getX(), (int)event.getY());
			lastMovePoint = downPoint;
			bMoveOperate = false;
			if (touchShapeOverlay != null && !isShapeOverlayNeedCheckDrag(touchShapeOverlay)){
				return false;
			};
			return true;
		}
		switch (event.getAction() & MotionEvent.ACTION_MASK) {
		case MotionEvent.ACTION_MOVE:
			synchronized (m_infoViews) {
				if (this.touchOverlay != null && touchOverlay.marker != null && this.touchOverlay.marker.isDraggable()) {
					// 满足长按条件，进入移动状态
					if(bMoveOperate || System.currentTimeMillis() - downTick > MARKER_LONG_PRESS_TIME)
					{
						moveInfoView(touchOverlay, event);
						if(!bMoveOperate)
						{
							map.onMarkerDragStart(touchOverlay.marker);
						}
						bMoveOperate = true;
						map.onMarkerDrag(touchOverlay.marker);
					}
					else
					{
						// 非移动状态时，手势移动出marker范围，手势无效
						Rect rect = new Rect();
						this.touchOverlay.getMarkerHitRect(rect);
						if (!rect.contains((int)event.getX(),(int) event.getY())) {
							this.touchOverlay = null;
							return false;
						}
					}
					return true;
				}
			}
			
			synchronized (m_shapeOverlays) {
				if (touchShapeOverlay != null && isShapeOverlayNeedCheckDrag(touchShapeOverlay)) {
					// 满足长按条件，进入移动状态
					if(bMoveOperate || System.currentTimeMillis() - downTick > MARKER_LONG_PRESS_TIME)
					{
						moveShapeOverlay(touchShapeOverlay, event);
						if(!bMoveOperate)
						{
							map.onShapeOverlayDragStart(touchShapeOverlay, new Point((int)event.getX(), (int)event.getY()));
						}
						bMoveOperate = true;
					}
					else
					{
						// 非移动状态时，手势移动大于一定距離范围，手势无效
						Point dPoint = new Point((int)event.getX() - downPoint.x, (int)event.getY() - downPoint.y);
						if (dPoint.x * dPoint.x + dPoint.y * dPoint.y >= (int)Math.pow(MOVE_GESTURE_BE_INVALID_PIXEL_DIS, 2)) {
							touchShapeOverlay = null;
							return false;
						}
					}
					return true;
				}
			}
			break;
		case MotionEvent.ACTION_UP:
			synchronized (m_infoViews) {
				if (this.touchOverlay != null) {
					upInfoView(touchOverlay, event);
					this.touchOverlay = null;
					bMoveOperate = false;
					return true;
				}
			}
			synchronized (m_shapeOverlays) {
				if (touchShapeOverlay != null) {
					upShapeOverlay(touchShapeOverlay, event);
					touchShapeOverlay = null;
					bMoveOperate = false;
					return true;
				}
			}
			break;
		default:
			break;
		}
		
		if(touchInfo != null)
		{
			// 点击在infowindow区域，先让子控件处理event
			return super.dispatchTouchEvent(event);
		}
		
		return false;
		
	}

	private boolean hitShapeOverlay(MotionEvent event) {
		if ((event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_DOWN) {
			touchShapeOverlay = null;
			synchronized (m_shapeOverlays) {
				for (KShapeOverlay overlay : m_shapeOverlays) {
					if (!overlay.isVisible()
							|| (!isShapeOverlayNeedCheckClick(overlay) && !isShapeOverlayNeedCheckDrag(overlay))) {
						continue;
					}
					
					synchronized (map.m_OverlayProcSync) {
						if(MapCore.selectOverlay(new Point((int)event.getX(), (int)(map.getViewHeight() - event.getY())), overlay.getOverlayId())){
							touchShapeOverlay = overlay;
							return true;
						}
					}
				}
			}
		}
		return false;
	}

	private boolean isShapeOverlayNeedCheckClick(KShapeOverlay overlay) {
		// 没有设置点击监听，不需要判断点击
		boolean bNeedCheck = false;
		if (overlay instanceof KArc) {
			bNeedCheck = (map.m_mapArcClickListener != null);
		} else if (overlay instanceof KCircle) {
			bNeedCheck = (map.m_mapCircleClickListener != null);
		} else if (overlay instanceof KGroundOverlay) {
			bNeedCheck = (map.m_mapGroundOverlayClickListener != null);
		} else if (overlay instanceof KPolyline) {
			bNeedCheck = (map.m_mapPolylineClickListener != null);
		} else if (overlay instanceof KPolygon) {
			bNeedCheck = (map.m_mapPolygonClickListener != null);
		}
		return bNeedCheck;
	}
	
	private boolean isShapeOverlayNeedCheckDrag(KShapeOverlay overlay) {
		// 没有设置overlay可拖拽，不需要判断拖拽
		boolean bNeedCheck = false;
		if (overlay instanceof KArc) {
			bNeedCheck = ((KArc)overlay).isDraggable();
		} else if (overlay instanceof KCircle) {
			bNeedCheck = ((KCircle)overlay).isDraggable();
		} else if (overlay instanceof KGroundOverlay) {
			bNeedCheck = ((KGroundOverlay)overlay).isDraggable();
		} else if (overlay instanceof KPolyline) {
			bNeedCheck = ((KPolyline)overlay).isDraggable();
		} else if (overlay instanceof KPolygon) {
			bNeedCheck = ((KPolygon)overlay).isDraggable();
		}
		return bNeedCheck;
	}

	//[internal] 最后添加的图层先处理hit响应
	public boolean hitInfoOverlay(MotionEvent event) {
		if ((event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_DOWN) {
			this.touchOverlay = null;
			this.touchInfo = null;
			
			// 先infowindow处理消息，避免marker在infowindow区域内marker先处理了消息
			synchronized (m_infoViews) {
				for (InfoView info : m_infoViews) {
					if(info.infoView == null || info.infoView.getVisibility() == View.GONE) {
						continue;
					}
					Rect infoRect = new Rect();
					info.getInfoHitRect(infoRect);
					if(infoRect.contains((int)event.getX(),(int) event.getY())) {
						this.touchInfo = info;
						return false;
					}
					else {
						break;
					}
				}
				
				for (InfoView overlay : m_infoViews) {
					if (overlay.marker == null || !overlay.marker.isVisible()
							|| ((map.m_markerClickListener == null || !overlay.marker.isClickable())&& !overlay.marker.isDraggable()) ) {
						//Log.i("LQ", "onMarkerClick onTouch hitInfoOverlay 0");
						continue;
					}
					//Log.i("LQ", "onMarkerClick onTouch hitInfoOverlay 1"+(!overlay.marker.isClickable()));
					
					// 窗口替代不响应消息
					if (m_infowindowSubtitude != null && overlay.marker.equals(m_infowindowSubtitude)) {
						continue;
					}
					
					Rect rect = new Rect();
					overlay.getMarkerHitRect(rect);
					if (rect.contains((int)event.getX(),(int) event.getY())) {
						this.touchOverlay = overlay;
					//	Log.i("LQ", "onMarkerClick onTouch hitInfoOverlay 2");
						return true;
					}
				}
			}
		}
		
		return false;
	}

	//[internal] 供外部刷新使用
	@Override
	public void onRefresh() {
		CopyOnWriteArrayList<InfoView> infoViews = map.getInfoViews();
		synchronized (infoViews) {
			for (InfoView info : infoViews) {
				info.recalcPos();
			}
		}
		invalidate();
	}

	
	@Override
	public boolean dispatchTouchEvent(MotionEvent event) {
		//Log.i(TAG, "dispatchTouchEvent" + event.getAction() + "/" + event);
		iDispatchTouchEventCheck = -1;
		return this.onTouchEvent(event);
	}

	@Override
	public void onPostRefresh() {
		postInvalidate();
	}
	
	public void onInfowindowRefresh(InfoView infoView)
	{
		if (infoView == null) {
			return;
		}
		
		CopyOnWriteArrayList<InfoView> infoViews = map.getInfoViews();
		synchronized (infoViews) {
			for (InfoView overlay : infoViews){
				if(!infoView.marker.getId().equals(overlay.marker.getId()) 
						&& overlay.marker.isInfoWindowShown())
				{
					overlay.marker.hideInfoWindow();
				}
			}
		}
	}
	
	public void moveInfoView(InfoView infoView,MotionEvent event) {
		Point point = new Point((int)event.getX(),(int)event.getY());
		KLatLng latLng = map.getProjection().fromScreenLocation(point);
		infoView.marker.setPosition(latLng);
	}
	
	public void upInfoView(InfoView infoView,MotionEvent event){
		//更新Marker坐标参数等状态。
		//计算当前的经纬度坐标。置为marker的坐标。
		if(!bMoveOperate)
		{
			if(infoView.marker != null && infoView.marker.isClickable()){
				
				map.onMarkerClick(infoView.marker);
			}
		}
		else
		{
			moveInfoView(infoView, event);
			map.onMarkerDragEnd(infoView.marker);
		}
	}
	
	private void upShapeOverlay(KShapeOverlay overlay,
			MotionEvent event) {
		Point point = new Point((int)event.getX(), (int)event.getY());
		if (!bMoveOperate) {
			map.onShapeOverlayClick(overlay, point);
		} else {
			moveShapeOverlay(overlay, event);
			map.onShapeOverlayDragEnd(overlay, point);
		}
		
	}
	private void moveShapeOverlay(KShapeOverlay overlay, MotionEvent event) {
		Point curPoint = new Point((int)event.getX(), (int)event.getY());
		if (Math.abs(curPoint.x - lastMovePoint.x) < 2 && Math.abs(curPoint.y - lastMovePoint.y) < 2) {
			return;
		}
		
		KLatLng lastLatLng = map.getProjection().fromScreenLocation(lastMovePoint);
		KLatLng curLatLng = map.getProjection().fromScreenLocation(curPoint);
		KLatLng dLatLng = new KLatLng(curLatLng.latitude - lastLatLng.latitude, curLatLng.longitude - lastLatLng.longitude);
		
		if (overlay instanceof KArc) {
			((KArc)overlay).move(dLatLng);
		} else if (overlay instanceof KCircle) {
			((KCircle)overlay).move(dLatLng);
		} else if (overlay instanceof KGroundOverlay) {
			((KGroundOverlay)overlay).move(dLatLng);
		} else if (overlay instanceof KPolyline) {
			((KPolyline)overlay).move(dLatLng);
		} else if (overlay instanceof KPolygon) {
			((KPolygon)overlay).move(dLatLng);
		}
		lastMovePoint = curPoint;
		map.onShapeOverlayDrag(touchShapeOverlay, curPoint);
	}

	public void reSort(){
		removeAllViews();
		addAllInfoView();
	}

	public void addAllInfoView(){
		CopyOnWriteArrayList<InfoView> infoViews = map.getInfoViews();
		synchronized (infoViews) {
			for (int i = 0; i < infoViews.size(); i++) {
				InfoView infoView = infoViews.get(i);	
				addView(infoView);
			}				
		}
	}

	public void setInfowindowSubtitude(KMarker infowindowSubstitute) {
		m_infowindowSubtitude = infowindowSubstitute;
	}

	public boolean singleTapUpOverlayClick(MotionEvent e) {
		synchronized (m_shapeOverlays) {
			if (touchShapeOverlay != null) {
				Point point = new Point((int)e.getX(), (int)e.getY());
				map.onShapeOverlayClick(touchShapeOverlay, point);
				touchShapeOverlay = null;
				return true;
			}
		}
		return false;
	}
}
