package com.cmmap.internal.maps.model;

import java.util.List;

import com.cmmap.api.maps.model.Arc;

/**
 * 弧形覆盖物类。
 * <p>它有以下属性：</p>
 * <ul>
 * <li>边框宽度<br>这个值设置后不会受到可视区域缩放级别的影响。</li>
 * <li>边框颜色<br>边框颜色是一个ARGB格式，ARGB请参考{@link android.graphics.Color}。默认为黑色。</li> 
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KArc extends KShapeOverlay{

	private KArcOptions arcOptions = null;
	private Arc m_parent = null;
	
	public void setArcOptions (KArcOptions arcOptions) {
		this.arcOptions = new KArcOptions(arcOptions);
		setVisible(arcOptions.isVisible());
		setLevel(arcOptions.getLevel());
		setZIndex(arcOptions.getZIndex());
	}
	
	public KArc() {
		setOverlayId(assignedId());
	}
	
	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId(){
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 返回弧形边框颜色，ARGB格式
	 * @return
	 */
	public int getStrokeColor(){
		return arcOptions.getStrokeColor();
	}
	
	/**
	 * 返回弧形边框宽度，单位：像素
	 * @return
	 */
	public float getStrokeWidth(){
		return arcOptions.getStrokeWidth();
	}

	/**
	 * 设置弧形边框颜色
	 * @param color
	 */
	public void setStrokeColor(int color) {
		arcOptions.strokeColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置弧形边框宽度
	 * @param width
	 */
	public void setStrokeWidth(float width) {
		arcOptions.strokeWidth(width);
		firePropertyChange(UPDATE_ALL);
	}
	
	public List<KLatLng> getPoints() {
		return this.arcOptions.getPoints();
	}
	
	@Override
	public void remove() {
		if (arcOptions != null) {			
			arcOptions.releaseResource();
			arcOptions = null;
		}
		m_parent = null;
		super.remove();
	}
	
	public void setParent(Arc arc) {
		m_parent = arc;
	}
	
	public Arc getParent() {
		return m_parent ;
	}

	public boolean isDraggable() {
		if (arcOptions != null) {			
			return arcOptions.isDraggable();
		}
		return false;
	}

	public void setDraggable(boolean enable) {
		if (arcOptions != null) {
			arcOptions.draggable(enable);			
		}
	}
	
	public void move(KLatLng dLatLng) {
		if (dLatLng == null || arcOptions == null || arcOptions.getPoints() == null) {
			return;
		}
		
		KLatLng start = arcOptions.getStart();
		KLatLng pass = arcOptions.getPassed();
		KLatLng end = arcOptions.getEnd();
		if (start == null || pass == null || end == null) {
			return;
		}
		arcOptions.points(new KLatLng(start.latitude + dLatLng.latitude, start.longitude + dLatLng.longitude),
							new KLatLng(pass.latitude + dLatLng.latitude, pass.longitude + dLatLng.longitude), 
							new KLatLng(end.latitude + dLatLng.latitude, end.longitude + dLatLng.longitude));
		firePropertyChange(UPDATE_ALL);
	}
}
