package com.cmmap.internal.maps.model;

import com.cmmap.api.maps.model.Circle;
import com.cmmap.internal.maps.KMapUtils;


/**
 * 圆形覆盖物类。
 * <p>它有有以下属性：</p>
 * <ul>
 * <li>圆心 <br>圆形圆心的经纬度。</li>
 * <li>半径<br>单位：米，这个值应该大于等于0。</li>
 * <li>边框宽度<br>这个值设置后不会受到地图域缩放级别的影响，单位：像素，默认为10。</li>
 * <li>边框颜色<br>边框的颜色，ARGB格式，参见{@link android.graphics.Color}，默认为黑色。</li>
 * <li>填充颜色<br>圆形填充的颜色，ARGB格式参见{@link android.graphics.Color}，默认为透明。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KCircle extends KShapeOverlay {
	
	private KCircleOptions circleOptions = null;
	private Circle m_parent = null;

	public void setCircleOptions(KCircleOptions options) {
		this.circleOptions = new KCircleOptions(options);
		setVisible(circleOptions.isVisible());
		setLevel(circleOptions.getLevel());
		setZIndex(circleOptions.getZIndex());
	}
	
	public KCircle() {
		setOverlayId(assignedId());
	}
	
	/**
	 * 判断圆形是否包含经纬度点
	 * @param point 经纬度点
	 * @return 是否包含 
	 */
	public boolean contains(KLatLng point)
	{
		float dis = KMapUtils.calculateLineDistance(point, getCenter());
		return Double.doubleToLongBits(dis) < Double.doubleToLongBits(getRadius());
	}
	
	/**
	 * 获取圆心坐标
	 * @return 圆心坐标
	 */
	public KLatLng getCenter(){
		return new KLatLng(circleOptions.getCenter());
	}
	
	public boolean isDottedLine(){
		return circleOptions.isDottedLine();
	}
	
	/**
	 * 获取圆的填充颜色
	 * @return
	 */
	public int getFillColor(){
		return circleOptions.getFillColor();
	}
	
	/**
	 * 获取圆的ID
	 * @return
	 */
	public String getId() {
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 返回圆半径：米
	 * @return
	 */
	public double getRadius(){
		return circleOptions.getRadius();
	}
	
	/**
	 * 返回圆的边框颜色
	 * @return
	 */
	public int getStrokeColor(){
		return circleOptions.getStrokeColor();
	}
	
	/**
	 * 获取边框宽度
	 * @return
	 */
	public float getStrokeWidth() {
		return circleOptions.getStrokeWidth();
	}
	
	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，参数必须大于等于0，如果为0则不会绘制边框
	 */
	public void setStrokeWidth(float width) {
		circleOptions.strokeWidth(width);
		firePropertyChange(UPDATE_ALL);
	}

	@Override
	public int hashCode() {
		return getOverlayId();
	}

	@Override 
	public boolean equals(Object object) {
		if (object == null 
				|| !(object instanceof KCircle)) {
			return false;
		}
		
		KCircle circle = (KCircle) object;
		return circle.getOverlayId() == getOverlayId();
	}
	
	/**
	 * 设置圆形的圆心经纬度坐标，参数不能为null，无默认值
	 * @param center  圆心坐标
	 */
	public void setCenter(KLatLng center) {
		this.circleOptions.center(center);
		firePropertyChange(UPDATE_LOCATION);
	}
	
	/**
	 * 设置填充颜色。
	 * @param color 填充颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setFillColor(int color) {
		this.circleOptions.fillColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置圆的半径：米
	 * @param radius
	 */
	public void setRadius(double radius) {
		this.circleOptions.radius(radius);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置边框颜色
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setStrokeColor(int color) {
		this.circleOptions.strokeColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	public boolean isDraggable() {
		if (circleOptions != null) {			
			return circleOptions.isDraggable();
		}
		return false;
	}

	public void setDraggable(boolean enable) {
		if (circleOptions != null) {
			circleOptions.draggable(enable);			
		}
	}
	
	@Override
	public void remove() {
		if (circleOptions != null) {
			circleOptions.releaseResource();
			circleOptions = null;			
		}
		m_parent = null;
		super.remove();
	}
	
	public void setParent(Circle circle) {
		m_parent = circle;
	}
	
	public Circle getParent() {
		return m_parent;
	}
	
	public void move(KLatLng dLatLng) {
		if (dLatLng == null || circleOptions == null || circleOptions.getCenter() == null) {
			return;
		}
		
		KLatLng center = circleOptions.getCenter();
		circleOptions.center(new KLatLng(center.latitude + dLatLng.latitude, center.longitude + dLatLng.longitude));
		firePropertyChange(UPDATE_ALL);
	}
}
