package com.cmmap.internal.maps.model;

import com.cmmap.api.maps.model.CircleOptions;

import android.graphics.Color;
import android.os.Parcel;
import android.os.Parcelable;


/**
 * 圆形覆盖物选项类。
 */
public class KCircleOptions{

	/**
	 * 圆心的坐标
	 */
	private KLatLng centerPos;
	
	/**
	 * 填充颜色
	 */
	private int fillColor = Color.argb(127, 0, 0, 127);	// 透明色
	
	/**
	 * 圆半径（米）
	 */
	private double radius = 100;
	
	/**
	 * 圆边框颜色
	 */
	private int strokeColor = Color.BLUE;
	
	
	/**
	 * 圆的边框宽度
	 */
	private float strokeWidth = 10;
	
	/**
	 * 是否可拖动
	 */
	private boolean draggable = false;
	
	
	/**
	 * level的值
	 */
	private int level = 0;
	
	/**
	 * Z轴的值
	 */
	private float zIndex = 0;
	
	
	/**
	 * 是否可见
	 */
	private boolean bVisible = true;
	
	private boolean mIsDottedLine = false;
	
	/**
	 * 默认构造
	 */
	public KCircleOptions() {
		centerPos = new KLatLng();
	}
	
	/**
	 * 构造函数
	 * @param options
	 */
	public KCircleOptions(KCircleOptions options){
		this.centerPos = new KLatLng(options.centerPos);
		this.fillColor = options.fillColor;
		this.radius = options.radius;
		this.strokeColor = options.strokeColor;
		this.strokeWidth = options.strokeWidth;
		this.draggable = options.draggable;
		this.level = options.level;
		this.zIndex = options.zIndex;
		this.bVisible = options.bVisible;
		this.mIsDottedLine = options.mIsDottedLine;
	}
	
	public KCircleOptions(CircleOptions options) {
		centerPos = new KLatLng(options.getCenter().latitude, options.getCenter().longitude);
		fillColor = options.getFillColor();
		radius = options.getRadius();
		strokeColor = options.getStrokeColor();
		strokeWidth = options.getStrokeWidth();
		draggable = options.isDraggable();
		level = options.getLevel();
		zIndex = options.getZIndex();
		bVisible = options.isVisible();
	}
	
	public KCircleOptions setDottedLine(boolean enable)
	{
		mIsDottedLine = enable;
		return this;
	}
	
	public boolean isDottedLine() 
	{
		return mIsDottedLine;
	}
	/**
	 * 设置圆心坐标。
	 * @param point 圆心坐标，参数不能为空
	 * @return 该对象本身
	 */
	public KCircleOptions center(KLatLng point){
		centerPos = new KLatLng(point);
		return this;
	}
	
	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public KCircleOptions fillColor(int color) {
		this.fillColor = color;
		return this;
	}
	
	/**
	 * 返回圆心坐标
	 * @return 圆点坐标
	 */
	public KLatLng getCenter() {
		return new KLatLng(this.centerPos);
	}
	
	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor()  {
		return this.fillColor;
	}
	
	/**
	 * 返回圆形的半径，单位米
	 * @return
	 */
	public double getRadius(){
		return this.radius;
	}
	
	/**
	 * 返回圆形边框颜色
	 * @return
	 */
	public int getStrokeColor() {
		return this.strokeColor;
	}
	
	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位像素
	 */
	public float getStrokeWidth(){
		return this.strokeWidth;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public KCircleOptions draggable(boolean enable) {
		draggable = enable;
		return this;
	}
	
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex(){
		return this.zIndex;
	}
	
	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible(){
		return this.bVisible;
	}
	
	/**
	 * 设置圆形半径。
	 * @param radius 圆形半径，单位：米，半径必须大于等于0
	 * @return 该对象本身
	 */
	public KCircleOptions radius(double radius){
		this.radius = radius;
		return this;
	}
	
	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public KCircleOptions strokeColor(int color) {
		this.strokeColor = color;
		return this;
	}
	
	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位像素，参数必须大于等于0，如果为0则不会绘制边框，默认：10。
	 * @return 该对象本身
	 */
	public KCircleOptions strokeWidth(float width) {
		this.strokeWidth = width;
		return this;
	}
	
	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */
	public KCircleOptions visible(boolean visible){
		this.bVisible = visible;
		return this;
	}
	
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KCircleOptions level(int level){
		this.level = level;
		return this;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public KCircleOptions zIndex(float zIndex) {
		this.zIndex = zIndex;
		return this;
	}

	public void releaseResource() {
		centerPos = null;
	}
}
