package com.cmmap.internal.maps.model;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.GroundOverlay;
import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.LatLngBounds;

/**
 * 图片覆盖物类。
 * <p>图片覆盖物层是一张图片以合适的大小贴在地图上，包含如下属性：</p>
 * <ul>
 * <li>图片显示范围<br>定义了图片显示的具体经纬度范围。</li>
 * <li>图片<br>给图片层的贴图。在添加图片层之前，如果没有设置图片，会报IllegalArgumentException 异常。</li>
 * <li>图片透明度<br>定义了图片的透明度，范围[0, 1.0]，0为不透明，1为全透明。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KGroundOverlay extends KShapeOverlay {
	
	private KGroundOverlayOptions options = null;
	private GroundOverlay m_parent = null;
	
	public KGroundOverlay() {
		setOverlayId(assignedId());
	}
	
	public void setGroundOverlayOptions (KGroundOverlayOptions opt) {
		options = new KGroundOverlayOptions(opt);
		setVisible(opt.isVisible());
		setLevel(opt.getLevel());
		setZIndex(opt.getZIndex());
	}
	
	public KGroundOverlayOptions getOptions()
	{
		return options;
	}
	
	/**
	 * 得到图片层的ID，一个地图上的每个图片层都有自己的ID
	 * @return
	 */
	public String getId()
	{
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 设置图片对象。
	 * @param image 图片对象
	 */
	public void setImage(BitmapDescriptor icon) {
		super.overlayImageRemove();
		options.image(icon);
		super.overlayImageChange();
	}
	
	/**
	 * 获取图片资源
	 * @return
	 */
	public BitmapDescriptor getImage() {
		return options.getImage();
	}
	/**
	 * 设置图片的坐标
	 * @param latLng
	 */
	public void setPosition(KLatLng latLng){
		options.position(latLng, options.getWidth(), options.getHeight());
		firePropertyChange(UPDATE_LOCATION);
		
	}
	
	/**
	 * 返回图片显示的的矩形区域 。
	 * @return 图片显示的的矩形区域 
	 */
	public LatLngBounds getBounds() {
		return options.getBounds();
	}

	/**
	 * 根据矩形区域设置图片显示的矩形区域。
	 * @param latLngBounds 矩形区域
	 */
	public void setPositionFromBounds(LatLngBounds latLngBounds) {
		if (latLngBounds != null && !latLngBounds.equals(options.getBounds())) {
			options.positionFromBounds(latLngBounds);
			firePropertyChange(UPDATE_ALL);
		}
	}

	/**
	 * 返回图片显示宽度。
	 * @return 图片显示宽度，单位：米
	 */
	public float getWidth() {
		return options.getWidth();
	}

	/**
	 * 返回图片显示高度。
	 * @return 图片显示高度，单位：米
	 */
	public float getHeight() {
		return options.getHeight();
	}

	/**
	 * 设置图片显示宽度，图片的高度根据图片自身的宽高比例自动变化。
	 * @param width 图片显示宽度，单位：米
	 */
	public void setDimensions(float width) {
		options.position(options.getLocation(), width);
	}

	/**
	 * 设置图片显示的宽度和高度，图片会被拉伸，不会保留之前的图片比例。
	 * @param width 图片显示宽度，单位：米
	 * @param height 图片显示高度，单位：米
	 */
	public void setDimensions(float width, float height) {
		options.position(options.getLocation(), width, height);
	}
	
	public float getBearing(){
		return options.getBearing();
	}
	
	public void setBearing(float bearing){
		options.bearing(bearing);
		firePropertyChange(UPDATE_ALL);
	}

	/**
	 * 返回绘制图片时的的透明度。
	 * @return 绘制图片时的的透明度，默认：0
	 */
	public float getTransparency() {
		return options.getTransparency();
	}
	
	public KLatLng getPosition(){
		return options.getLocation();
	}
	
	/**
	 * 设置图片透明度
	 * @param transparency
	 */
	public void setTransparency(float transparency){
		options.transparency(transparency);
		firePropertyChange(UPDATE_ALL);
	}
	
	public boolean isDraggable() {
		if (options != null) {			
			return options.isDraggable();
		}
		return false;
	}

	public void setDraggable(boolean enable) {
		if (options != null) {
			options.draggable(enable);			
		}
	}
	
	/**
	 * 删除
	 * @return
	 */
	public void remove(){
		if (options != null) {
			options.releaseResource();
			options = null;
		}
		m_parent = null;
		super.remove();
	}
	
	public void setParent(GroundOverlay groundOverlay) {
		m_parent = groundOverlay;
	}
	
	public GroundOverlay getParent() {
		return m_parent;
	}
	
	public void move(KLatLng dLatLng) {
		if (dLatLng == null || options == null || options.getBounds() == null) {
			return;
		}
		LatLngBounds bounds = options.getBounds();
		LatLng northeast = new LatLng(bounds.northeast.latitude + dLatLng.latitude, bounds.northeast.longitude + dLatLng.longitude);
		LatLng southwest = new LatLng(bounds.southwest.latitude + dLatLng.latitude, bounds.southwest.longitude + dLatLng.longitude);
		options.positionFromBounds(new LatLngBounds(northeast, southwest));
		firePropertyChange(UPDATE_ALL);
	}
}
