package com.cmmap.internal.maps.model;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.GroundOverlayOptions;
import com.cmmap.api.maps.model.LatLngBounds;


public class KGroundOverlayOptions{
	
	private float m_anchorU = 0.5f;	// 锚点x位置
	private float m_anchorV = 0.5f;	// 锚点y位置
	private float m_bearing = 0;	// 旋转角度
	private LatLngBounds m_bounds = null;	// 显示范围
	private BitmapDescriptor m_descriptor = null; // 图片
	private KLatLng m_location = null;	// 中心坐标
	private float m_width = 100;  // 米
	private float m_height = 100;  // 米
	private boolean m_visible = true;	// 是否显示
	private float m_transparent = 0.0f;	// 图片透明度
	private boolean m_draggable = false; // 是否可拖拽
	private int level = 0;
	private float m_z_index = 0.0f;		// z坐标
	
	public KGroundOverlayOptions(KGroundOverlayOptions options)
	{
		this.m_anchorU = options.m_anchorU;
		this.m_anchorV = options.m_anchorV;
		this.level = options.level;
		this.m_z_index = options.m_z_index;
		this.m_visible = options.m_visible;
		this.m_bearing = options.m_bearing;
		this.m_transparent = options.m_transparent;
		this.m_descriptor = options.m_descriptor;
		this.m_draggable = options.m_draggable;
		
		// 锚点方法和设置矩形方法互斥
		if (options.getBounds() != null)
		{
			this.m_bounds = options.getBounds();
			this.m_location = null;
		}else
		{
			this.m_bounds = null;
			this.m_location = new KLatLng(options.getLocation());
			this.m_width = options.m_width;
			this.m_height = options.m_height;
		}
	}
	
	public KGroundOverlayOptions (GroundOverlayOptions options) {
		level = options.getLevel();
		m_z_index = options.getZIndex();
		m_visible = options.isVisible();
		m_transparent = options.getTransparency();
		m_descriptor = options.getImage();
		m_bounds = options.getBounds();
		m_draggable = options.isDraggable();
	}
	
	public KGroundOverlayOptions () {
		
	}
	
	
	/**
	 * 设置图片的对齐方式，[0,0]是左上角，[1,1]是右下角 。如果不设置，默认为[0.5,0.5]图片的中心点。
	 * @param u 在宽度（水平方向）上的对齐方式，范围为[0,1]。
	 * @param v 在高度（垂直方向）上的对齐方式，范围为[0,1]。
	 * @return
	 */
	public KGroundOverlayOptions anchor(float u, float v){
		this.m_anchorU = u;
		this.m_anchorV = v;
		return this;
	}
	
	/**
	 * 图片层从正北顺时针的角度，相对锚点旋转。
	 * @param bearing 图片层从正北顺时针的角度，范围为[0,360)。
	 * @return
	 */
	public KGroundOverlayOptions bearing(float bearing) {
		this.m_bearing = bearing;
		return this;
	}
	
	/**
	 * 从左边开始，水平方向的对齐方式，范围为[0,1]。
	 * @return 水平方向对齐返回的值。
	 */
	public float getAnchorU() {
		return m_anchorU;
	}
	
	/**
	 * 从上边开始，垂直方向的对齐方式，范围为[0,1]。
	 * @return 垂直方向对齐返回的值。
	 */
	public float getAnchorV() {
		return m_anchorV;
	}
	
	/**
	 * 得到为此GroundOverlayOptions 设置的角度。
	 * @return 图片层的角度。
	 */
	public float getBearing(){
		return this.m_bearing;
	}
	
	public LatLngBounds getBounds() {
		return m_bounds;
	}
	
	/**
	 * GroundOverlayOptions 的宽 。
	 * @return 图片层的宽。
	 */
	public float getWidth() {
		return m_width;
	}
	
	/**
	 * 得到GroundOverlayOptions 的高。
	 * @return 图片层的高。
	 */
	public float getHeight() {
		return m_height;
	}
	
	/**
	 * 得到GroundOverlayOptions 的图片 。
	 * @return GroundOverlayOptions 的图片 。
	 */
	public BitmapDescriptor getImage() {
		return m_descriptor;
	}
	
	/**
	 * GroundOverlayOptions 的锚点位置。
	 * @return 此图片层的锚点。如果用矩形区域设置图片层位置，此方法返回null。
	 */
	public KLatLng getLocation() {
		return this.m_location;
	}
	
	/**
	 * 得到GroundOverlayOptions 的透明度。
	 * @return 图片层的透明度。
	 */
	public float getTransparency() {
		return this.m_transparent;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public KGroundOverlayOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}
		
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	
	/**
	 * GroundOverlayOptions 的z轴指数。
	 * @return 图片层的z轴指数。
	 */
	public float getZIndex(){
		return this.m_z_index;
	}
	
	/**
	 * 指定此图片层的图片。
	 * @param image 图片层使用的图片。
	 * @return 根据新图片返回的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions image(BitmapDescriptor image) {
		if (image != null){
			this.m_descriptor = image;
		}
		return this;
	}
	
	/**
	 * 得到图片层是否可见。
	 * @return 图片层是否可见，true 可见，false 不可见。
	 */
	public boolean isVisible() {
		return this.m_visible;
	}
	
	/**
	 * 根据锚点和宽高设置图片层。在显示时，图片高度根据图片的比例自动匹配。
	 * @param location 图片层的锚点。
	 * @param width 图片层的宽，单位：米。
	 * @return 根据锚点设置的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions position(KLatLng location, float width) {
		return position(location, width, width * m_descriptor.getHeight() / m_descriptor.getWidth());
	}
	
	/**
	 * 根据锚点和宽高设置图片层。在显示时，图片会被缩放来适应指定的尺寸。
	 * @param location 图片层的锚点。
	 * @param width 图片层的宽，单位：米。
	 * @param height 图片层的高，单位：米。
	 * @return 根据锚点设置的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions position(KLatLng location, float width, float height) {
		if (location != null)
		{
			this.m_bounds = null;
			this.m_width = width;
			this.m_height = height;
			this.m_location = location;
		}
		return this;
	}
	
	public KGroundOverlayOptions positionFromBounds(LatLngBounds bounds) {
		if (bounds != null) {
			this.m_bounds = bounds;
			this.m_location = null;
		}
		return this;
	}
	
	/**
	 * 设置图片层的透明度。默认透明度为0，不透明。
	 * @param transparency 图片层的透明度，范围为[0,1]，0为不透明，1为全透明。
	 * @return 根据新设置的透明度返回的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions transparency(float transparency) {
		this.m_transparent = transparency;
		return this;
	}
	
	/**
	 * 设置图片层是否可见。默认为可见 。
	 * @param visible true 可见，false 不可见。
	 * @return 新设置后的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions visible(boolean visible){
		this.m_visible = visible;
		return this;
	}
	
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KGroundOverlayOptions level(int level){
		this.level = level;
		return this;
	}
	
	/**
	 * 设置图片层的z轴指数。
	 * @param zIndex z轴指数。
	 * @return 新设置z轴指数后的GroundOverlayOptions。
	 */
	public KGroundOverlayOptions zIndex(float zIndex) {
		this.m_z_index = zIndex;
		return this;
	}

	void releaseResource() {
		m_descriptor = null;
		m_bounds = null;
		m_location = null;
	}
}
