package com.cmmap.internal.maps.model;

import java.io.OptionalDataException;
import java.util.ArrayList;
import java.util.List;

import android.graphics.BitmapFactory.Options;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.Marker;

/**
 * 标记覆盖物类。
 * <p>标记覆盖物是在地图上绘制一个带图标的点。一个标记覆盖物有如下属性：</p>
 * <ul>
 * <li>锚点<br>图标摆放在地图上的基准点。默认情况下，锚点是从图片下沿的中间处。</li>
 * <li>位置<br>Marker通过经纬度的值来标注在地图上的。</li>
 * <li>标题<br>默认的信息窗口的显示标题，随时可以更改。</li>
 * <li>描述<br>默认的信息窗口的显示描述，随时可以更改。</li>
 * <li>图标<br>Marker显示的图标。如果未设置图标，API将使用默认的图标。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KMarker extends KInfoOverlay {
	
	private Marker parent = null;
	
	private KMarkerOptions markerOptions = null;
	
	/**
	 * Marker的附加对象
	 */
	private Object dataObject = null;
		
	/**
	 * infowindow是否显示
	 */
	private boolean bInfowindowVisible = false;
	
	private int bitmapIndex = 0;
	
	
	public void setMarkerOptions(KMarkerOptions markerOptions) {
		this.markerOptions = new KMarkerOptions(markerOptions);
		setVisible(markerOptions.isVisible());
		setLevel(markerOptions.getLevel());
		setZIndex(markerOptions.getZIndex());
	}
	
	public int getBitmapIndex() {
		return bitmapIndex;
	}
	
	public void changeIndex()
	{
		bitmapIndex = (bitmapIndex + 1) % markerOptions.getIcons().size();
		firePropertyChange(UPDATE_ALL);
	}
	
	public KMarkerOptions getMarkerOptions()
	{
		return this.markerOptions;
	}
	
	public KMarker() {
		setOverlayId(assignedId());
		bitmapIndex = 0;
		markerOptions = new KMarkerOptions();
	}
	
	/**
	 * 删除当前marker并销毁Marker的图片等资源
	 */
	public void destroy() {
		remove();
	}
	

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (o == null) {
			return false;
		}
		if (!(o instanceof KMarker)) {
			return false;
		}
		return getOverlayId() == ((KMarker) o).getOverlayId();
	}
	
	/**
	 * 返回图标列表，如果用户仅设置了一个图标，则返回一个图标。
	 * @return 图标列表
	 */
	public ArrayList<BitmapDescriptor> getIcons() {
		ArrayList<BitmapDescriptor> icons = markerOptions.getIcons();
		if(icons == null)
		{
			icons = new ArrayList<BitmapDescriptor>(1);
			icons.add(markerOptions.getIcon());
		}
		return icons;
	}
	
	/**
	 * @return 返回marker 的id，每个marker 的唯一标识，用来区分不同的marker
	 */
	public String getId(){
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * @return Marker的附加对象
	 */
	public Object getObject(){
		return this.dataObject;
	}
	
	/**
	 * 得到多少帧刷新一次图片资源。默认为20，最小为1
	 * @return 刷新周期，速度越小速度越快。默认为20，最小为1。
	 */
	public int getPeriod(){
		return markerOptions.getPeriod();
	}
	
	/**
	 * @return Marker的经纬度坐标
	 */
	public KLatLng getPosition(){
		return markerOptions.getPosition();
	}
	
	/**
	 * 返回 Marker图片旋转的角度，从正北开始，逆时针计算(reserve)
	 * @return 
	 */
	public float getRotateAngle() {
		return markerOptions.getRotateAngle();
	}
	
	/**
	 * @return 返回当前marker 的文字片段
	 */
	public String getSnippet() {
		
		return markerOptions.getSnippet();
	}
	
	/**
	 * @return marker 标题
	 */
	public String getTitle(){
		return markerOptions.getTitle();
	}
	

	@Override
	public int hashCode() {
		return getOverlayId();
	}
	
	/**
	 * 隐藏marker的InfoWindow信息窗口
	 * 如果marker本身是不可以见的，此方法将不起任何作用
	 */
	public void hideInfoWindow() {
		if(isVisible() && bInfowindowVisible)
		{
			bInfowindowVisible = false;
			super.infoWindowShowStatusChanged();
		}
	}
	
	@Override
	public void setZIndex(float zIndex) {
		super.setZIndex(zIndex);
		KOverlayManager.getInstance().updateTopIndex(zIndex);
	}

	/**
	 * 获得标记的拖动状态
	 * @return
	 */
	public boolean isDraggable() {
		return markerOptions.isDraggable();
	}
	
	/**
	 * 返回marker是否是平贴在地图上
	 * @return
	 */
	public boolean isFlat(){
		return markerOptions.isFlat();
	}
	
	/**
	 * marker的信息窗口是否显示
	 * @return
	 */
	public boolean isInfoWindowShown(){
		return bInfowindowVisible;
	}
	
	/**
	 * 返回标记是否设置为近大远小效果
	 * @return
	 */
	public boolean isPerspective() {
		return markerOptions.isPerspective();
	}
	
	@Override
	public void remove() {
		hideInfoWindow();
		if (markerOptions != null) {
			markerOptions.recycleIcon();
			markerOptions = null;
		}
		super.remove();
		dataObject = null;
		setParent(null);
	}
	
	/**
	 * 设置当前marker的锚点。 锚点是定位图标接触地图平面的点。图标的左上顶点为（0,0）点，右下点为（1,1）点。默认情况下，锚点为（0.5,1.0）。
	 * @param anchorU 锚点水平范围的比例
	 * @param anchorV 锚点垂直范围的比例
	 */
	public void setAnchor(float anchorU, float anchorV) {
		if (anchorU != markerOptions.getAnchorU() || anchorV != markerOptions.getAnchorV()) {
			markerOptions.anchor(anchorU, anchorV);
			firePropertyChange(UPDATE_ALL);
		}
	}
	
	/**
	 * 返回锚点在水平范围的比例。
	 * @return 锚点在水平范围的比例
	 */
	public float getAnchorU(){
		return markerOptions.getAnchorU();
		
	}
	
	/**
	 * 返回锚点垂直范围的比例。
	 * @return 锚点在垂直范围的比例
	 */
	public float getAnchorV(){
		return markerOptions.getAnchorV();
	}
	
	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽
	 */
	public void setDraggable(boolean paramBoolean) {
		markerOptions.draggable(paramBoolean);
	}
	
	
	/**
	 * 设置是否平贴地图显示。
	 * @param flat 是否平贴地图
	 */
	public void setFlat(boolean flat) {
		if(markerOptions.isFlat() != flat)
		{
			markerOptions.setFlat(flat);
			firePropertyChange(UPDATE_ALL);
		}
	}
	
	/**
	 * 设置单个图标。
	 * @param icon 图标
	 */
	public void setIcon(BitmapDescriptor icon) {
		super.overlayImageRemove();
		markerOptions.icon(icon);
		super.overlayImageChange();
	}
	
	/**
	 * 设置Marker动画的每一帧图片，动画的描点和大小以第一帧为准，建议图片大小保持一致
	 * @param icons
	 */
	public void setIcons(List<BitmapDescriptor> icons) {
		super.overlayImageRemove();
		markerOptions.icons(icons);
		super.overlayImageChange();
	}
	
	/**
	 * 设置Marker的附加对象
	 * @param object
	 */
	public void setObject(java.lang.Object object){
		this.dataObject = object;
	}
	
	/**
	 * 设置多少帧刷新一次图片资源，Marker动画的间隔时间，值越小动画越快
	 * @param period
	 */
	public void setPeriod(int period) {
		if(markerOptions.getPeriod() != period)
		{
			markerOptions.period(period);
			super.overlayImageChange();
		}
	}
	
	/**
	 * 设置标记的近大远小效果，用在marker初始化之后
	 * @param isPerspective
	 */
	public void setPerspective(boolean isPerspective) {
		markerOptions.perspective(isPerspective);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置当前显示的经纬度点。 
	 * @param latlng 当前显示的经纬度点
	 */
	public void setPosition(KLatLng latlng) {
		this.markerOptions.setGps(true);
		if(!markerOptions.getPosition().equals(latlng))
		{
			this.markerOptions.position(latlng);
			positionChanged();
		}
	}
	
	/**
	 * 设置Maker固定显示在屏幕上的像素坐标位置。
	 * <br>若用此方式，marker固定显示在屏幕上；如果想让标记随地图移动，可以使用{@link #setPosition} 改变。
	 * @param x 横向像素点，相对于地图视图的左上角
	 * @param y 纵向像素点，相对于地图视图的左上角
	 */
	public void setPositionByPixels(int x, int y) {
		if(markerOptions.isGps() || markerOptions.getX() != x || markerOptions.getY() != y)
		{
			this.markerOptions.setGps(false);
			this.markerOptions.setX(x);
			this.markerOptions.setY(y);
			super.positionChanged();
		}
	}

	/**
	 * 设置图片旋转角度。
	 * @param angle 图片旋转角度，从正北开始，逆时针计算
	 */
	public void setRotateAngle(float rotate) {
		if(Math.abs(rotate - markerOptions.getRotateAngle()) >= 1.0) {
			markerOptions.rotateAngle(rotate);
			super.markerRotateAngleChanged();
		}
	}
	
	/**
	 * 设置 描述信息。
	 * @param snippet 描述信息
	 */
	public void setSnippet(java.lang.String snippet) {
		markerOptions.snippet(snippet);
		super.infoWindowContentChanged();
	}
	
	/**
	 * 设置marker标题
	 * @param title
	 */
	public void setTitle(java.lang.String title){
		markerOptions.title(title);
		super.infoWindowContentChanged();
	}
	

	/**
	 * 设置当前marker在最上面显示。
	 */
	public void setToTop() {
		setZIndex(KOverlayManager.getInstance().generateTopZIndex());
	}
	
	@Override
	public void setVisible(boolean isVisible) {
		if(!isVisible && this.isInfoWindowShown())
		{
			this.hideInfoWindow();
		}
		
		if(isVisible() != isVisible)
		{
			super.setVisible(isVisible);
		}
	}
	

	/**
	 * 显示当前Marker信息窗口。
	 */
	public void showInfoWindow() {
		if(isVisible() && !bInfowindowVisible)
		{
			bInfowindowVisible = true;
			super.infoWindowShowStatusChanged();
		}
	}
	
	public void setParent(Marker mark) {
		parent = mark;
	}
	
	public Marker getParent() {
		return parent;
	}
	
	public void stopAnimates() {
		if (markerOptions.isAnimatesDrop()) {
			super.stopMarkerAnimates();
		}
	}

	/**
	 * 设置是否可点击。
	 * @param enable 是否可点击
	 */
	public void setClickable(boolean enable) {
		markerOptions.clickable(enable);		
	}
	
	/**
	 * 获得标记的点击状态
	 * @return
	 */
	public boolean isClickable() {
		return markerOptions.isClickable();
	}
	

}
