package com.cmmap.internal.maps.model;

import java.util.List;

/**
 * 箭头覆盖物类。
 * <p>拥有以下属性：</p>
 * <ul>
 * <li>顶点<br>箭头是由多个连贯的点构成的。</li>
 * <li>宽度<br>单位是像素，默认为10。</li>
 * <li>颜色<br>颜色是ARGB格式，参考{@link android.graphics.Color}，默认是黑色。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KNavigateArrow extends KShapeOverlay{

	private KNavigateArrowOptions navigateArrowOptions = null;
	
	
	public void setNavigateArrowOptions (KNavigateArrowOptions arrowOptions) {
		this.navigateArrowOptions = new KNavigateArrowOptions(arrowOptions);
		setVisible(arrowOptions.isVisible());
		setZIndex(arrowOptions.getZIndex());
		setLevel(arrowOptions.getLevel());
	}
	
	public KNavigateArrow() {
		setOverlayId(assignedId());
	}
	
	/**
	 * 返回箭头对象的id
	 * @return 箭头对象的id
	 */
	public String getId(){
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 返回当前箭头的侧边颜色
	 * @return 箭头的侧边颜色
	 */
	public int getSideColor(){
		return navigateArrowOptions.getSideColor();
	}
	
	/**
	 * 返回当前箭头的顶颜色
	 * @return 箭头的侧边颜色
	 */
	public int getTopColor(){
		return navigateArrowOptions.getTopColor();
	}
	
	/**
	 * 返回箭头宽度
	 * @return 箭头宽度，单位：像素，默认：10像素
	 */
	public float getWidth(){
		return navigateArrowOptions.getWidth();
	}
	
	/**
	 * 设置当前箭头的侧边颜色
	 * @param color 箭头的侧边颜色
	 */
	public void setSideColor(int color) {
		navigateArrowOptions.sideColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置当前箭头的顶颜色
	 * @param color 箭头的顶颜色
	 */
	public void setTopColor(int color) {
		navigateArrowOptions.topColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置箭头宽度
	 * @param width 箭头宽度，单位：像素
	 */
	public void setWidth(float width) {
		navigateArrowOptions.width(width);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 返回顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改返回的顶点坐标列表不会影响对象内部的坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<KLatLng> getPoints() {
		return this.navigateArrowOptions.getPoints();
	}
	
	/**
	 * 设置箭头的顶点
	 * 如果调用此方法后修改了List的值将不会影响到箭头
	 * @param points
	 */
	public void setPoints(java.util.List<KLatLng> points){
		navigateArrowOptions.addAll(points);
		firePropertyChange(UPDATE_ALL);
	}
	
	@Override
	public void remove() {
		if (navigateArrowOptions != null) {
			navigateArrowOptions.releaseResource();
			navigateArrowOptions = null;
		}
		super.remove();
	}
	
}
