package com.cmmap.internal.maps.model;

import java.util.concurrent.atomic.AtomicInteger;

import android.content.Context;

import com.cmmap.internal.driver.base.GlobalSettings;


/**
 * 自定义的图形类基类
 *
 */
abstract public class KOverlay implements Comparable<KOverlay>  {
	private int level = 0;
	private float zIndex;
	private boolean isVisible = true;
	private KOverlayListener overlayListener;
	private static final float FLOAT_ACCURACY = 0.000001f;
	
	public static final int UPDATE_VISIBLE = 1;
	public static final int UPDATE_LOCATION = 2;
	public static final int UPDATE_LEVEL = 6;
	public static final int UPDATE_ZINDEX = 8;
	public static final int UPDATE_ADD_POINT = 64;
	public static final int UPDATE_ALL = 0xffffffff;

	
	//[internal] 每个图形都有id
	private int id;
	private static final AtomicInteger a_id = new AtomicInteger();

	protected void setOverlayId(int id) {
		this.id = id;
	}

	public int getOverlayId() {
		return id;
	}

	protected static int assignedId(){
		return a_id.incrementAndGet();
	}
	protected boolean checkLocation(KLatLng location) {
		if (location != null && location.isValidLatlon())
			return true;
		return false;
	}
	
	@Override
	public int compareTo(KOverlay another) {
		// marker覆盖物比较，ZIndex越大则优先度越高，ZIndex相同时，后添加的优先度越高
		if (getZIndex() - another.getZIndex() > FLOAT_ACCURACY) {
			return -1;
		} else if (another.getZIndex() - getZIndex() > FLOAT_ACCURACY) {
			return 1;
		} else {
			int iRet = another.getOverlayId() > getOverlayId() ? 1 : -1;
			return iRet;
		}
	}
	
	/**
	 * 获取显示层级（主要用于处理覆盖物与文字的压盖关系）
	 */
	public int getLevel() {
		
		return level;
	}
	
	
	/**
	 * 设置显示层级（主要用于处理覆盖物与文字的压盖关系）
	 */
	public void setLevel(int level) {
		if (level != this.level) {
			this.level = level;
			if (overlayListener != null) {
				overlayListener.updateLevel(id);
			}
		}
	}
	
	
	
	/**
	 * 获取显示顺序。
	 * 
	 * @return 显示顺序
	 */
	public float getZIndex() {
		return zIndex;
	};

	/**
	 * 设置顺序。
	 * 
	 * @param zIndex 显示顺序
	 *            
	 */
	public void setZIndex(float zIndex) {
		if (zIndex != this.zIndex) {
			this.zIndex = zIndex;
			if (overlayListener != null) {
				overlayListener.updateZIndex(id);
			}
		}
	};

	/**
	 * 获取是否显示
	 * 
	 * @return 是否显示
	 */
	public boolean isVisible() {
		return this.isVisible;
	};

	/**
	 * 设置是否显示。
	 * 
	 * @param isVisible
	 *            
	 */
	public void setVisible(boolean isVisible) {
		this.isVisible = isVisible;
		if (overlayListener != null) {
			overlayListener.firePropertyChange(id, UPDATE_VISIBLE);
		}
	};

	/**
	 * 删除。
	 */
	public void remove() {
		if (overlayListener != null) {
			overlayListener.remove(id);
			overlayListener = null;
		}
	};

	protected Context getContext() {
		return GlobalSettings.getInstance().getContext();
	}

	//[internal]有改变通知刷新
	protected void firePropertyChange(int updateMask) {
		if (overlayListener != null) {
			overlayListener.firePropertyChange(id, updateMask);
		}
	}
	
	protected void overlayImageRemove() {
		if(overlayListener != null) {
			overlayListener.overlayImageRemove(id);
		}
	}
	
	protected void overlayImageChange() {
		if(overlayListener != null) {
			overlayListener.overlayImageChange(id);
		}
	}
	
	protected void clearCache() {
		if (overlayListener != null) {
			overlayListener.clearCache(id);
		}
	}
	
	public void setPropertyChangeListener(KOverlayListener listener){
		this.overlayListener = listener;
	}
	
	protected void infoWindowShowStatusChanged() {
		if(overlayListener != null) {
			((KMarkerListener)overlayListener).InfoWindowShowStatusChanged(id);
		}
	}
	
	protected void markerRotateAngleChanged(){
		if(overlayListener != null) {
			((KMarkerListener)overlayListener).RotateAngleChanged(id);
		}
	}
	
	protected void infoWindowContentChanged() {
		if(overlayListener != null) {
			((KMarkerListener)overlayListener).InfoWindowContentChanged(id);
		}
	}
	
	protected void positionChanged() {
		if(overlayListener != null) {
			((KMarkerListener)overlayListener).PositionChanged(id);
		}
	}
	
	protected void stopMarkerAnimates() {
		if (overlayListener != null) {
			((KMarkerListener)overlayListener).stopMarkerAnimates(id);
		}
	}
	
}
