package com.cmmap.internal.maps.model;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.atomic.AtomicInteger;

import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.os.Environment;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.BitmapDescriptorFactory;
import com.cmmap.api.maps.model.TileProvider;
import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.base.Tools;
import com.cmmap.internal.maps.KMapInitializer;
import com.cmmap.internal.maps.KMapView;

/**
 * 内部overlay管理类，不对外提供
 *
 */
public class KOverlayManager {
	private static final KOverlayManager INSTANCE = new KOverlayManager();
	
	private static final AtomicInteger overlayIdAssigner = new AtomicInteger();
	
	private HashMap<String, KOverlay> m_mapOverlay = new HashMap<String, KOverlay>();
	
	private HashMap<String, TileProvider> m_mapTileProvider = new HashMap<String, TileProvider>();
	
	private float m_topZIndex = 0;
	
	public static String strTileDistCachePath = null;
	
	public Bitmap defaultMarker = null;
	
	private HashMap<Object, OnOverlayListener> m_mapOverlayListener = new HashMap<Object, KOverlayManager.OnOverlayListener>();
	
	// 内部构造函数，不对外提供
	KOverlayManager() {
		
	}
	
	public static KOverlayManager getInstance() {
		return INSTANCE;
	}
	
	public static int assignId() {
		return overlayIdAssigner.incrementAndGet();
	}
	
	public void clear() {
		m_mapOverlay.clear();
	}
	
	public void addOverlay(String id, KOverlay overlay, Object obj, OnOverlayListener listener) {
		m_mapOverlay.put(id, overlay);
		m_mapOverlayListener.put(obj, listener);
	}
	
	public KOverlay findOverlay(String id) {
		return m_mapOverlay.get(id);
	}
	
	public void removeOverlay(String id) {
		m_mapOverlay.remove(id);
		// 所有overlay移除后，释放marker默认图片资源
		if (m_mapOverlay.size() <= 0 && defaultMarker != null) {
			defaultMarker.recycle();
			defaultMarker = null;
		}
	}
	
	public void removeOverlay(Object obj) {
		if (obj != null) {
			OnOverlayListener listener = m_mapOverlayListener.get(obj);
			if (listener != null) {
				listener.onRemove(obj);
			}
			m_mapOverlayListener.remove(obj);
		}
	}
	
	public float generateTopZIndex() {
		return m_topZIndex + 0.1f;
	}
	
	public void updateTopIndex(float zIndex) {
		if (Float.floatToIntBits(zIndex) > Float.floatToIntBits(m_topZIndex)) {
			m_topZIndex = zIndex;
		}
	}
	
	public void addTileProvider(KTileOverlay tileOverlay) {
		m_mapTileProvider.put(tileOverlay.getId(), tileOverlay.m_Options.getTileProvider());
	}
	
	public TileProvider getTileProvider(String id) {
		return m_mapTileProvider.get(id);
	}
	
	public void removeTileProvider(String id) {
		m_mapTileProvider.remove(id);
	}
	
	public static String getDefaultTileDiskCachePath() {
		if (strTileDistCachePath == null) {
			String strSuffixPath = "/cmmap/tilecache";
			try {
				PackageInfo  packageInfo = GlobalSettings.getInstance().getContext().getPackageManager().getPackageInfo(GlobalSettings.getInstance().getContext().getPackageName(), 64);
				strSuffixPath = "/" + packageInfo.packageName + "/tilecache";
			} catch (NameNotFoundException e1) {
				e1.printStackTrace();
			}
			
			// 获取外置sd卡路径
			Tools tools = new Tools(GlobalSettings.getInstance().getApplicationContext());
			ArrayList<String> arrMountList = tools.getMountPoints();
			boolean bFind = false;
			if (arrMountList != null && arrMountList.size() > 0) {
				// 遍历挂载点路径，找到可用的路径
				for(String path : arrMountList) {
					String strTmp = path + strSuffixPath;
					File tmpfile = new File(strTmp);
					if (tmpfile.exists() || tmpfile.mkdirs()) {
						strTileDistCachePath = strTmp;
						bFind = true;
						break;
					}
				}
			} 
			// 没有获取到外置sd卡路径，设置内置sd卡或内存路径
			if (!bFind) {
				strTileDistCachePath = GlobalSettings.getInstance().getContext().getCacheDir().getAbsolutePath();
				if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
					strTileDistCachePath = Environment.getExternalStorageDirectory() + strSuffixPath;
				}
				File file = new File(strTileDistCachePath);
				if(!file.exists()){
					file.mkdirs();
				}
			}
			
		}
		return strTileDistCachePath;
	}
	
	BitmapDescriptor getDefaultMarkerIcon() {
		if (defaultMarker == null) {
			BitmapDescriptor srcDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/default_marker.png");
			Bitmap src = srcDescriptor.getBitmap();
			
			// 按照dpi缩放默认图片
			float scale = KMapInitializer.getInstance().getDPI() / 320;
			Matrix matrix = new Matrix();
			matrix.postScale(scale, scale);
			defaultMarker = Bitmap.createBitmap(src,0,0,src.getWidth(),src.getHeight(),matrix,true);
		}
		return BitmapDescriptorFactory.fromBitmap(defaultMarker);
	}
	
	public static interface OnOverlayListener {
		
		void onRemove(Object overlay);
	}
}
