package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.api.maps.model.Polygon;
import com.cmmap.internal.mapcore.MapCore;


/**
 * 多边形覆盖物类。
 * <p>一个多边形可以是凸面体，也可以是凹面体,它有以下属性：</p>
 * <ul>
 * <li>边框<br>一个多边开的边框由一系列的顺时针或逆时针顶点组成。边框不需要声明起点和终点一致。如果边框的起点与终点不一致，API会自动将它封闭。</li>
 * <li>边框宽度<br>边框的宽度以像素为单位，不会受可视区域缩放级别的影响。默认为10。</li>
 * <li>边框颜色<br>边框颜色是一个ARGB格式，ARGB请参考android.graphics.Color。默认为黑色。</li>
 * <li>填充颜色<br>填充颜色是一个ARGB格式，ARGB格式可参考android.graphics.Color。默认是透明的。如果多边形的几何图形不正确，则填充颜色不会被绘制。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KPolygon extends KShapeOverlay {
	private KPolygonOptions options = null;
	private Polygon m_parent = null;
	
	/**
	 * 判断多边形是否包含传入的经纬度点
	 * @param point 传入的经纬度点
	 * @return 是否包含
	 * @return 
	 */
	public boolean contains(KLatLng point)  {
		return MapCore.isPtInPoly(getPoints(), point);
	}
	
	/**
	 * 返回填充颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getFillColor() {
		return options.getFillColor();
	}
	
	public void setPolygonOptions (KPolygonOptions options) {
		this.options = new KPolygonOptions(options);
		setVisible(options.isVisible());
		setLevel(options.getLevel());
		setZIndex(options.getZIndex());
	}
	
	public KPolygon() {
		setOverlayId(assignedId());
	}
	
	/**
	 * 返回多边形的Id。
	 * @return
	 */
	public String getId()  {
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 返回多边形顶点的列表
	 * @return
	 */
	public List<KLatLng> getPoints() {
		return options.getPoints();
	}
	
	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		return options.getStrokeColor();
	}
	
	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth(){
		return options.getStrokeWidth();
	}
	
	/**
	 * 设置多边形填充颜色。
	 * @param color
	 */
	public void setFillColor(int color) {
		options.fillColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改这个顶点坐标列表不会影响对象内部的坐标列表。
	 * @param points 顶点坐标列表
	 */
	public void setPoints(java.util.List<KLatLng> points)  {
		options.latLngs.clear();
		for (int i = 0; i < points.size(); i++) {
			options.add(points.get(i));
		}
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置边框颜色。
	 * @param color 边框颜色，ARGB格式，参见{@link android.graphics.Color}
	 */
	public void setStrokeColor(int color)   {
		options.strokeColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素
	 */
	public void setStrokeWidth(float width)  {
		options.strokeWidth(width);
		firePropertyChange(UPDATE_ALL);
	}
	
	public boolean isDraggable() {
		if (options != null) {			
			return options.isDraggable();
		}
		return false;
	}

	public void setDraggable(boolean enable) {
		if (options != null) {
			options.draggable(enable);			
		}
	}
	
	@Override
	public void remove() {
		if (options != null) {
			options.releaseResource();
			options = null;
		}
		m_parent = null;
		super.remove();
	}
	
	public void setParent(Polygon polygon) {
		m_parent = polygon;
	}
	
	public Polygon getParent() {
		return m_parent;
	}
	
	public void move(KLatLng dLatLng) {
		if (dLatLng == null || options == null || options.getPoints() == null) {
			return;
		}
		
		List<KLatLng> points = options.getPoints();
		ArrayList<KLatLng> newPoints = new ArrayList<KLatLng>(points.size());
		for (int i = 0; i < points.size(); i++) {
			KLatLng pt = new KLatLng(points.get(i).latitude + dLatLng.latitude, points.get(i).longitude + dLatLng.longitude);
			newPoints.add(pt);
		}
		options.addAll(newPoints);
		firePropertyChange(UPDATE_ALL);
	}
}
