package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.Polyline;

/**
 * 线段覆盖物类。
 * <p>一个线段是多个连贯点的集合线段，它拥有以下属性：</p>
 * <ul>
 * <li>顶点<br>线段是由两个顶点之间连贯的点构成的。如果两个顶点相同，则一个线段将闭合。</li>
 * <li>宽度<br>宽度是单位是像素。宽度是可视区域的缩放级别无关。默认为10。</li>
 * <li>颜色<br>线段的颜色是ARGB格式，颜色格式可以参考android.graphics.Color。默认是黑色(0xff000000)。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 * <p>覆盖物的操作必须主线程中调用，否则会抛出IllegalStateException。</p>
 */
public class KPolyline extends KShapeOverlay {
	
	private KPolylineOptions options = null;
	private Polyline m_parent = null;
	
	/**
	 * 构建函数
	 * @return 
	 */
	public KPolyline(){
		setOverlayId(assignedId());
	}		
	
	/**
	 * 设置选项
	 * @return 
	 */
	public void setPolyLineOptions (KPolylineOptions options) {
		this.options = new KPolylineOptions(options);
		setVisible(options.isVisible());
		setLevel(options.getLevel());
		setZIndex(options.getZIndex());
	}
	
	/**
	 * 获取选项
	 * @return 
	 */
	public KPolylineOptions getPolyLineOptions() {
		return options;
	}	
		
	/**
	 * 返回线的颜色。
	 * @return 填充颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor(){
		return options.getColor();
	}
	
	/**
	 * 返回覆盖物ID。
	 * <br>ID是该覆盖物在所属的地图对象里的唯一标识。
	 * @return ID 覆盖物ID
	 */
	public String getId() {
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 获取最近形点
	 * @param lat
	 * @return
	 */
	public KLatLng getNearestLatLng(KLatLng lat) {
		return null;
	}
	
	/**
	 * 返回顶点坐标列表。
	 * <br>这个坐标点列表与对象内部保存的顶点坐标列表不是同一个对象，所以，用户修改返回的顶点坐标列表不会影响对象内部的坐标列表。
	 * @return 顶点坐标列表
	 */
	public List<KLatLng> getPoints() {
		return options.getPoints();
	}	
	
	/**
	 * 返回线段的宽度值
	 * @return
	 */
	public float getWidth()  {
		return options.getWidth();
	}
	
	/**
	 * 返回线宽。
	 * @return 线宽，单位：像素，默认：10像素
	 */
	public boolean isDottedLine(){
		return options.isDottedLine();
	}
	
	/**
	 * 线段是否为大地曲线，默认false，不画大地曲线
	 * @return
	 */
	public boolean isGeodesic(){
		return options.isGeodesic();
	}
	
	/**
	 * 设置名称是否可见。
	 * @param visible 是否可见
	 */
	public void setLineNameVisible(boolean visible) {
		options.setLineNameVisible(visible);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 返回名称是否可见。
	 * @return visible 是否可见
	 */
	public boolean getLineNameVisible() {
		return options.getLineNameVisible();
	}
	
	/**
	 * 返回名称前景色。
	 * @return 颜色，默认：黑色
	 */
	public int getNameColor() {
		return options.getNameColor();
	}
	
	/**
	 * 设置名称前景色。
	 * @param 颜色，默认：黑色
	 * @return 无
	 */
	public void nameColor(int color) {
		options.nameColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 返回名称边缘色。
	 * @return 颜色，默认：不色
	 */
	public int getNameEdgeColor() {
		return options.getNameEdgeColor();
	}
	
	/**
	 * 设置名称背景色。
	 * @param 颜色，默认：黑色
	 * @return 无
	 */
	public void nameEdgeColor(int color) {
		options.nameEdgeColor(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 返回名称大小。
	 * @return 字体大小，默认：22
	 */
	public int getNameFontSize() {
		return options.getNameFontSize();
	}
	
	/**
	 * 设置名称大小。
	 * @param 字体大小，默认：22
	 * @return 无
	 */
	public void nameFontSize(int size) {
		options.nameFontSize(size);
		firePropertyChange(UPDATE_ALL);
	}
	
	
	/**
	 * 删除
	 * @return
	 */
	public void remove(){
		if (options != null) {
			options.ReleaseResource();
			options = null;			
		}
		super.remove();
		m_parent = null;
	}
	
	/**
	 * 设置线段的颜色
	 * @param color
	 */
	public void setColor(int color) {
		options.color(color);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 获取线段的颜色列表
	 * @return 颜色列表
	 */
	public List<Integer> getColorValues() {
	    return options.getColorValues();
	}
	
	/**
	 * 获取线段纹理列表
	 * @return 纹理列表
	 */
	public List<BitmapDescriptor> getCustomTextureList(){
        return options.getCustomTextureList();
    }
	
	/**
	 * 获取线段纹理index列表
	 * @return 分段纹理index列表
	 */
	public List<java.lang.Integer> getCustomTextureIndex(){
        return options.getCustomTextureIndex();
    }
	
	/**
	 * 获取线段名称列表
	 * @return 名称列表
	 */
	public List<String> getLineNameList(){
        return options.getLineNameList();
    }
	
	/**
	 * 获取线段名称index列表
	 * @return 分段名称index列表
	 */
	public List<java.lang.Integer> getLineNameIndex(){
        return options.getLineNameIndex();
    }
	
	/**
	 * 设置线段的颜色
	 * @param colors 颜色列表
	 * @return
	 */
	public void colorValues(List<Integer> colors) {
		options.colorValues(colors);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置线段纹理list
	 * @param bitmapDescriptors 纹理列表
	 * @return
	 */
	public void setCustomTextureList(List<BitmapDescriptor> bitmapDescriptors) {
        options.setCustomTextureList(bitmapDescriptors);
        firePropertyChange(UPDATE_ALL);
    }
	
	/**
	 * 设置线段纹理index数组
	 * @param integers 每一段对应的纹理，用纹理列表的index来对应
	 * @return
	 */
	public void setCustomTextureIndex(List<Integer> integers) {
        options.setCustomTextureIndex(integers);
        firePropertyChange(UPDATE_ALL);
    }
	
	/**
	 * 设置线段名称list
	 * @param lstName 名称列表
	 * @return
	 */
	public void setLineNameList(List<String> lstName) {
        options.setLineNameList(lstName);
        firePropertyChange(UPDATE_ALL);
    }
	
	/**
	 * 设置线段名称index数组
	 * @param integers 每一段对应的名称，用名称列表的index来对应
	 * @return
	 */
	public void setLineNameIndex(List<Integer> integers) {
        options.setLineNameIndex(integers);
        firePropertyChange(UPDATE_ALL);
    }
	
	/**
	 * 设置是否画虚线，默认为false，画实线。
	 * @param isDottedLine
	 */
	public void setDottedLine(boolean isDottedLine) {
		options.setDottedLine(isDottedLine);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置是否大地曲线
	 * @param geodesic
	 */
	public void setGeodesic(boolean geodesic) {
		options.geodesic(geodesic);	
		firePropertyChange(UPDATE_ALL);
	}
	
	
	/**
	 * 设置线段顶点
	 * @param points
	 */
	public void setPoints(java.util.List<KLatLng> points) {
		options.addAll(points);
		firePropertyChange(UPDATE_ALL);
	}
	
	/**
	 * 设置线宽
	 * @param width
	 */
	public void setWidth(float width) {
		options.width(width);
		firePropertyChange(UPDATE_ALL);
	}
	
	public boolean isDraggable() {
		if (options != null) {			
			return options.isDraggable();
		}
		return false;
	}

	public void setDraggable(boolean enable) {
		if (options != null) {
			options.draggable(enable);			
		}
	}
	
	public void setParent(Polyline polyline) {
		m_parent = polyline;
	}
	
	public Polyline getParent() {
		return m_parent;
	}
	
	public void move(KLatLng dLatLng) {
		if (dLatLng == null || options == null || options.getPoints() == null) {
			return;
		}
		
		List<KLatLng> points = options.getPoints();
		ArrayList<KLatLng> newPoints = new ArrayList<KLatLng>(points.size());
		for (int i = 0; i < points.size(); i++) {
			KLatLng pt = new KLatLng(points.get(i).latitude + dLatLng.latitude, points.get(i).longitude + dLatLng.longitude);
			newPoints.add(pt);
		}
		options.addAll(newPoints);
		firePropertyChange(UPDATE_ALL);
	}
}
